/* ClearPulse-80057.kts */

datasource MultiAnodePmt 
{
    int base_address = 0x00f00000;
    
    VmeCrate crate;
    VmeController controller("SBS-810");
    VmeModule module("ClearPulse_80057", "PmtDataProcessor");
    SoftwareModule readout_timer("IntervalTimer");
    
    crate.installController(controller);
    crate.installModule(module, base_address);

    on trigger(readout_timer) {
        module.read();
    }

    on run_end {
        // read time counter from the DP moudle //
        DataRecord RunTimeRecord;
        Register realtime_reg, livetime_reg;
        module.readRegister(0x24, realtime_reg);
        module.readRegister(0x28, livetime_reg);
        RunTimeRecord.fill("RealTime", realtime_reg);
        RunTimeRecord.fill("LiveTime", livetime_reg);
        RunTimeRecord.send();
    }

    ////// Choose <<ONE>> configuration from examples below //////

    // PMT Module (80190) Software Trigger //
    if (1) {
        SoftwareModule trigger_timer("IntervalTimer");
        on run_begin {
            readout_timer.setInterval(0, 10000);
            trigger_timer.setInterval(1, 0);
            module.resetDpFifo();
        }
        on trigger(trigger_timer) {
            module.issuePmtTrigger();
            echo("software PMT trigger");
        }
    }

    // DP Module (80057) Software Trigger //
    if (0) {
        SoftwareModule trigger_timer("IntervalTimer");
        on run_begin {
            readout_timer.setInterval(0, 10000);
            trigger_timer.setInterval(1, 0);

            module.enablePmtExtTrigIn();
            module.enablePmtExtTrigOut();
            module.enableDpExtTrigIn();
            module.enableDpExtTrigOut();

            module.resetDpFifo();
        }
        on trigger(trigger_timer) {
            module.issueDpTrigger();
            echo("software DP trigger");
        }
    }

    // Dynode/Anode Trigger //
    if (0) {
        on run_begin {
            readout_timer.setInterval(0, 10000);

            module.setDpBusyOn();
            for (int ch = 0; ch < 64; ch++) {
                module.setPmtAnodeThreshold(ch, 300);
                module.setPmtAnodeTriggerChannelEnable(ch);
            }
            module.setPmtGainLow();
            module.setPmtDynodeThreshold(11);
            module.setPmtHoldTime(120);

            module.enablePmtExtTrigIn();
            module.enablePmtExtTrigOut();
            module.enableDpExtTrigIn();
            module.enableDpExtTrigOut();

            module.enablePmtDynodeTrigger();
            //module.enablePmtAnodeTrigger();

            module.resetDpFifo();
            module.setDpBusyOff();
        }
    }
}
