/* DataProcessor03.cc */
/* Sample program to read indexed data */


#include <iostream>
#include <iomanip>
#include "MushArgumentList.hh"
#include "KinokoDataSource.hh"
#include "KinokoDataSection.hh"
#include "KinokoIndexedDataSection.hh"
#include "KinokoDataProcessor.hh"
#include "KinokoStandaloneComponent.hh"


using namespace std;


class TMyDataConsumer: public TKinokoDataConsumer {
  public:
    TMyDataConsumer(string DataSourceName, string SectionName);
    virtual ~TMyDataConsumer();
    virtual void OnRunBegin(void);
    virtual void OnReceiveDataPacket(void* DataPacket, long PacketSize);
  protected:
    string _DataSourceName, _SectionName;
    int _DataSourceId, _SectionId;
    TKinokoIndexedDataSectionScanner* _Scanner;
};



TMyDataConsumer::TMyDataConsumer(string DataSourceName, string SectionName)
{
    _DataSourceName = DataSourceName;
    _SectionName = SectionName;

    _DataSourceId = -1;
    _SectionId = -1;

    _Scanner = 0;
}

TMyDataConsumer::~TMyDataConsumer()
{
}

void TMyDataConsumer::OnRunBegin(void)
{
    // get DataSource object by name//
    TKinokoDataSource* DataSource = _InputDataDescriptor->DataSource(_DataSourceName);

    if (DataSource != 0) {
	// get DataSourceId //
	_DataSourceId = DataSource->DataSourceId();

	// get DataSection object by name //
	TKinokoDataSection* DataSection = DataSource->DataSection(_SectionName);

	if (DataSection != 0) {
	    // get SectionId //
	    _SectionId = DataSection->SectionId();

	    // check section type and get scanner object //
	    if (DataSection->SectionType() != TKinokoDataSection::SectionType_Indexed) {
		throw TKinokoException("invalid section type");
	    }
	    _Scanner = ((TKinokoIndexedDataSection*) DataSection)->Scanner();
	}
    }
}

void TMyDataConsumer::OnReceiveDataPacket(void* DataPacket, long PacketSize)
{
    // get DataSourceId //
    int DataSourceId = TKinokoDataStreamScanner::DataSourceIdOf(DataPacket);
    if (DataSourceId != _DataSourceId) {
	return;
    }

    int SectionId = TKinokoDataSectionScanner::SectionIdOf(DataPacket);
    if (SectionId != _SectionId) {
	return;
    }

    int NumberOfElements = _Scanner->NumberOfElements(DataPacket);
    int Address, Data;
    for (int Index = 0; Index < NumberOfElements; Index++) {
	_Scanner->ReadFrom(DataPacket, Index, Address, Data);
	cout << Address << " " << Data << endl;
    }
}


int main(int argc, char** argv)
{
    TMushArgumentList ArgumentList(argc, argv);

    TMyDataConsumer MyDataConsumer("CamacAdc", "adc");
    TKinokoStandaloneDataConsumer StandaloneDataConsumer(&MyDataConsumer, "MyDataConsumer");

    try {
	StandaloneDataConsumer.Start(ArgumentList);
    }
    catch (TKinokoException &e) {
	cerr << "ERROR: " << e << endl;
    }

    return 0;
}
