/* KameNtuple.cc */
/* Created by Enomoto Sanshiro on 29 June 2009. */
/* Last updated by Enomoto Sanshiro on 8 November 2011. */


#include <string>
#include "KameNtuple.hh"

using namespace std;
using namespace kame;


TKameNtuple::TKameNtuple(void)
{
    _LastRowIndex = 0;
    _LastSegmentIndex = 0;
}

TKameNtuple::TKameNtuple(unsigned NumberOfColumns)
: TKameTable(NumberOfColumns)
{
    _LastRowIndex = 0;
    _LastSegmentIndex = 0;
}

TKameNtuple::TKameNtuple(const TKameNtuple& Ntuple)
: TKameTable(Ntuple), _Tree(Ntuple._Tree)
{
    _ColumnTypeList = Ntuple._ColumnTypeList;
    _SegmentBoundaryList = Ntuple._SegmentBoundaryList;
    _LastRowIndex = Ntuple._LastRowIndex;
    _LastSegmentIndex = Ntuple._LastSegmentIndex;
}

TKameNtuple::~TKameNtuple()
{
}

TKameNtuple& TKameNtuple::operator=(const TKameNtuple& Ntuple)
{
    if (this == &Ntuple) {
        return *this;
    }

    TKameTable::operator=(Ntuple);
    _Tree = Ntuple._Tree;

    _ColumnTypeList = Ntuple._ColumnTypeList;
    _SegmentBoundaryList = Ntuple._SegmentBoundaryList;
    _LastRowIndex = Ntuple._LastRowIndex;
    _LastSegmentIndex = Ntuple._LastSegmentIndex;

    return *this;
}


void TKameNtuple::SetColumnProperties(unsigned ColumnIndex, const std::string& ColumnName, const std::string& ColumnType)
{
    TKameTable::AddColumn(ColumnName, ColumnIndex);

    if (! ColumnType.empty()) {
        while (_ColumnTypeList.size() < ColumnIndex) {
            _ColumnTypeList.push_back("");
        }
        _ColumnTypeList.push_back(ColumnType);
    }
}

const std::vector<std::string>& TKameNtuple::ColumnTypeList(void) const
{
    return _ColumnTypeList;
}

void TKameNtuple::BreakSegment(void)
{
    _SegmentBoundaryList.push_back(NumberOfRows());
}

unsigned TKameNtuple::NumberOfSegments(void) const
{
    return _SegmentBoundaryList.size();
}

unsigned TKameNtuple::SegmentIndexOf(unsigned RowIndex) const
{
    unsigned SegmentIndex = (
	(RowIndex >= _LastRowIndex) ? _LastSegmentIndex : 0
    );

    while (SegmentIndex < _SegmentBoundaryList.size()) {
	if (RowIndex < _SegmentBoundaryList[SegmentIndex]) {
	    break;
	}
	SegmentIndex++;
    }

    _LastRowIndex = RowIndex;
    _LastSegmentIndex = SegmentIndex;

    return SegmentIndex;
}

void TKameNtuple::Dump(std::ostream& os)
{
    os << "### TREE CONTENT ###" << endl;
    _Tree.Dump(os);
    os << endl;

    os << "### TABLE CONTENT ###" << endl;

    if (! _ColumnTypeList.empty()) {
        os << "# ColumnTypes: ";
        for (unsigned i = 0; i < _ColumnTypeList.size(); i++) {
            if (i != 0) {
                os << "  ";
            }
            os << _ColumnTypeList[i];
        }
        os << endl;
    }

    TKameTable::Dump(os);
}
