/* KameNtuple.hh */
/* Created by Enomoto Sanshiro on 29 June 2009. */
/* Last updated by Enomoto Sanshiro on 8 November 2011. */

#ifndef __KameNtuple_hh__
#define __KameNtuple_hh__


#include <string>
#include <vector>
#include "KameTree.hh"
#include "KameTable.hh"


namespace kame {


class TKameNtuple: public TKameTable {
  public:
    TKameNtuple(void);
    explicit TKameNtuple(unsigned NumberOfColumns);
    TKameNtuple(const TKameNtuple& Ntuple);
    virtual ~TKameNtuple();
    virtual TKameNtuple& operator=(const TKameNtuple& Ntuple);
  public:
    inline TKameTable::TRow operator[](unsigned RowIndex);
    inline TKameTree& operator[](const std::string& Key);
    inline TKameTree& Tree(void);
    inline TKameTable& Table(void);
  public:
    inline const TKameTable::TRow operator[](unsigned RowIndex) const;
    inline const TKameTree& operator[](const std::string& Key) const;
    inline const TKameTree& Tree(void) const;
  public:
    virtual void SetColumnProperties(unsigned Index, const std::string& Name = "", const std::string& Type = "");
    virtual const std::vector<std::string>& ColumnTypeList(void) const;
    virtual void BreakSegment(void);
    virtual unsigned NumberOfSegments(void) const;
    virtual unsigned SegmentIndexOf(unsigned RowIndex) const;
    virtual void Dump(std::ostream& os = std::cout);
  private:
    mutable TKameTree _Tree;
  private:
    std::vector<std::string> _ColumnTypeList;
    std::vector<unsigned> _SegmentBoundaryList;
    mutable unsigned _LastRowIndex, _LastSegmentIndex;
};



inline TKameTable::TRow TKameNtuple::operator[](unsigned RowIndex)
{
    return TKameTable::operator[](RowIndex);
}

inline TKameTree& TKameNtuple::operator[](const std::string& Key)
{
    return _Tree[Key];
}

inline TKameTree& TKameNtuple::Tree(void) 
{ 
    return _Tree; 
}

inline TKameTable& TKameNtuple::Table(void) 
{ 
    return *this; 
}

inline const TKameTable::TRow TKameNtuple::operator[](unsigned RowIndex) const
{
    return TKameTable::operator[](RowIndex);
}

inline const TKameTree& TKameNtuple::operator[](const std::string& Key) const
{
    return _Tree[Key];
}

inline const TKameTree& TKameNtuple::Tree(void) const
{ 
    return _Tree; 
}


}

// suger for convenient includes //
#include "KameTableFormat.hh"


#endif
