/* KamePlainTextRepository.hh */
/* Created by Enomoto Sanshiro on 1 July 2009. */
/* Last updated by Enomoto Sanshiro on 23 June 2011. */


#ifndef __KamePlainTextRepository_hh__
#define __KamePlainTextRepository_hh__


#include <string>
#include <map>
#include <cstdio>
#include "KameTree.hh"
#include "KameNtuple.hh"
#include "KameTableFormat.hh"
#include "KameRepository.hh"


namespace kame {

    
class TKamePlainTextRepository: public TKameRepository, public TKameTableFormat {
  public:
    TKamePlainTextRepository(void);
    TKamePlainTextRepository(const std::string& RepositoryName);
    virtual ~TKamePlainTextRepository();
    virtual void EnableCompression(void);
  public:
    virtual void SaveNtuple(const TKameNtuple& Ntuple, const std::string& Name);
    virtual void LoadNtuple(TKameNtuple& Ntuple, const std::string& Name, int Revision=-1);
  protected:
    virtual std::string FileNameOf(const std::string& ObjectName, int Revision = -1);
    virtual std::istream* OpenInputFile(const std::string& ObjectName, int Revision = -1);
    virtual std::ostream* OpenOutputFile(const std::string& ObjectName, int Revision = -1);
    virtual void CloseInputFile(std::istream* File);
    virtual void CloseOutputFile(std::ostream* File);
  protected:
    std::string _RepositoryName;
    bool _IsCompressionEnabled;
  private:
    bool _IsRepositoryCreated;
    std::map<std::string, int> _RevisionTable;
    std::map<std::ios*, std::FILE*> _CommandPipeTable;
    std::map<std::ios*, std::ios*> _MyFileTable;
};

}
#endif
