/* KameTable.cc */
/* Created by Enomoto Sanshiro on 29 June 2009. */
/* Last updated by Enomoto Sanshiro on 29 June 2009. */


#include <string>
#include <sstream>
#include <iomanip>
#include <map>
#include "KameTable.hh"


using namespace std;
using namespace kame;


TKameTable::TKameTable(void)
{
    _NumberOfRows = 0;
}

TKameTable::TKameTable(unsigned NumberOfColumns)
: _Table(NumberOfColumns)
{
    _NumberOfRows = 0;
    for (unsigned Index = 0; Index < NumberOfColumns; Index++) {
        ostringstream os;
        os << setfill('0') << setw(3) << Index;
        _ColumnNameList.push_back(os.str());
    }
}

TKameTable::TKameTable(const TKameTable& Table)
{
    if (this != &Table) {
	_NumberOfRows = Table._NumberOfRows;
	_Table = Table._Table;
	_ColumnNameList = Table._ColumnNameList;
    }
}

TKameTable& TKameTable::operator=(const TKameTable& Table)
{
    if (this != &Table) {
        _NumberOfRows = Table._NumberOfRows;
        _Table = Table._Table;
        _ColumnNameList = Table._ColumnNameList;
    }

    return *this;
}

unsigned TKameTable::AddColumn(const std::string& ColumnName, int ColumnIndex) 
{
    unsigned Index = (ColumnIndex < 0) ? _Table.size() : ColumnIndex;

    if (Index >= _Table.size()) {
	_Table.insert(
	    _Table.end(), Index - _Table.size() + 1, 
	    std::vector<TKameVariant>(_NumberOfRows)
	);
    }

    while (_ColumnNameList.size() <= Index) {
        ostringstream os;
        os << setfill('0') << setw(3) << _ColumnNameList.size();
        _ColumnNameList.push_back(os.str());
    }
    if (! ColumnName.empty()) {
        _ColumnNameList[Index] = ColumnName;
    }

    _ColumnNameIndexTable[_ColumnNameList[Index]] = Index;

    return Index;
}

unsigned TKameTable::AddRow(int RowIndex) 
{
    unsigned Index = (RowIndex < 0) ? _NumberOfRows : RowIndex;

    if (Index >= _NumberOfRows) {
	_NumberOfRows = Index + 1;
	for (unsigned i = 0; i < _Table.size(); i++) {
	    _Table[i].resize(_NumberOfRows);
	}
    }
    
    return Index;
}

void TKameTable::Clear(void)
{
    for (unsigned i = 0; i < _Table.size(); i++) {
	_Table[i].erase(_Table[i].begin(), _Table[i].end());
    }
    _NumberOfRows = 0;
}

void TKameTable::Dump(std::ostream& os)
{
    if (NumberOfColumns()) {
        os << "# ColumnNames: ";
        for (unsigned i = 0; i < NumberOfColumns(); i++) {
            if (i != 0) {
                os << "  ";
            }
            os << '"' << ColumnNameList()[i] << '"';
        }
    }
    os << endl << endl;

    for (unsigned i = 0; i < NumberOfRows(); i++) {
	for (unsigned j = 0; j < NumberOfColumns(); j++) {
            if (j != 0) {
                os << "  ";
            }
            TKameVariant& Value = this->operator[](i)[j];
            if (Value.IsVoid()) {
                os << "\"\"";
            }
            else if (Value.IsString()) {
                os << '"' << this->operator[](i)[j] << '"';
            }
            else {
                os << this->operator[](i)[j];
            }
	}
	os << endl;
    }
}
