/* KameTableFormatProcessor.hh */
/* Created by Enomoto Sanshiro on 1 July 2009. */
/* Last updated by Enomoto Sanshiro on 8 November 2011. */


#ifndef __KameTableFormatProcessor_hh__
#define __KameTableFormatProcessor_hh__


#include <iostream>
#include <string>
#include <vector>
#include "KameNtuple.hh"
#include "KameTreeBuilder.hh"


namespace kame {


class TKameTableFormatLineExtractor {
  public:
    TKameTableFormatLineExtractor(std::istream& Input, char LineTerminator = '\0');
    virtual ~TKameTableFormatLineExtractor();
    virtual bool GetNext(std::string& Line);
  protected:
    std::istream& _Input;
    char _LineTerminator;
  private:
    char _LastChar;
    int _CRCount;
};


class TKameTableFormatTokenizer {
  public:
    TKameTableFormatTokenizer(void);
    virtual ~TKameTableFormatTokenizer();
    virtual char SetDelimiter(char Delimiter);
    virtual char SetQuote(char Quote);
    virtual void Tokenize(const std::string& Line, std::vector<std::string>& ElementList) const;
  protected:
    char _Delimiter, _Quote;
};


class TKameTableFormatColumnParser: public TKameTableFormatTokenizer {
  public:
    TKameTableFormatColumnParser(void);
    virtual ~TKameTableFormatColumnParser();
    virtual void SetColumnTypeList(const std::vector<std::string>& ColumnTypeList);
    virtual unsigned ProcessLine(TKameTable::TRow Row, std::string& Line, unsigned ColumnOffset);
  protected:
    enum TColumnType {
	ColumnType_Int,
	ColumnType_Float,
	ColumnType_String,
	_NumberOfColumnTypes
    };
    std::vector<TColumnType> _ColumnTypeList;
};


class TKameTableFormatHeaderProcessor: public TKameTableFormatTokenizer {
  public:
    TKameTableFormatHeaderProcessor(void);
    virtual ~TKameTableFormatHeaderProcessor(void);
    virtual void EnableCsvHeader(void);
    virtual void SetCommentHeader(char CommentHeader);
    virtual void SetTreeIndent(char TreeIndent);
  public:
    virtual void SetStorage(TKameTree* Tree);
    virtual bool ProcessFirstLine(const std::string& Line);
    virtual bool ProcessLine(const std::string& Line);
  protected:
    virtual bool ProcessCsvHeader(const std::string& Line);
    virtual bool ProcessRootTreeDescriptor(const std::string& Line);
  protected:
    TKameTreeBuilder* _TreeBuilder;
    bool _IsCsvHeaderEnabled;
    char _CommentHeader, _TreeIndent;
};

}
#endif
