/* KameTreeBuilder.cc */
/* Created by Enomoto Sanshiro on 8 November 2011. */
/* Last updated by Enomoto Sanshiro on 4 April 2012 */

#include <iostream>
#include <sstream>
#include <string>
#include <deque>
#include "KameTree.hh"
#include "KameJsonParser.hh"
#include "KameTreeBuilder.hh"


using namespace std;
using namespace kame;



TKameTreeBuilder::TKameTreeBuilder(TKameTree* Tree)
{
    _IsInlineJsonEnabled = true;

    _CurrentDepth = -1;
    _LastNode = 0;
    _CurrentParentStack.push_back(Tree);
}

TKameTreeBuilder::~TKameTreeBuilder()
{
}

void TKameTreeBuilder::DisableInlineJson(void)
{
    _IsInlineJsonEnabled = false;
}

void TKameTreeBuilder::AddNode(int Depth, const std::string& Key, const std::string& Value)
{
    if (_CurrentDepth < 0) {
        // initial depth, can be non-zero
        _CurrentDepth = Depth;
    }

    if (Depth > _CurrentDepth) {
	_DepthStepStack.push_back(Depth - _CurrentDepth);
	_CurrentDepth = Depth;
	_CurrentParentStack.push_back(_LastNode);
    }
    else if (Depth < _CurrentDepth) {
	while (_CurrentDepth > max(Depth, 0)) {
	    _CurrentDepth -= _DepthStepStack.back();
	    _DepthStepStack.pop_back();
	    _CurrentParentStack.pop_back();
	}
	if (Depth != _CurrentDepth) {
	    throw TKameException("bad indent structure");
	}
    }

    // make empty-string void-value //
    TKameVariant VariantValue;
    if (! Value.empty()) {
        VariantValue = Value;
    }
    _LastNode = &_CurrentParentStack.back()->AppendNode(Key);

    if ((! Value.empty()) && _IsInlineJsonEnabled) {
        istringstream Line(Value.c_str());
        TKameTree JsonTree;
        TKameJsonParser JsonParser;
        try {
            JsonParser.Parse(Line, JsonTree);
            *_LastNode = JsonTree;
        }
        catch (TKameException &e) {
            _LastNode->Value() = VariantValue;
        }
    }
    else {
        _LastNode->Value() = VariantValue;
    }
}

void TKameTreeBuilder::AppendLine(const std::string& Line)
{
    if (_LastNode) {
        _LastNode->Value() = _LastNode->As<string>() + '\n' + Line;
    }
}
