/* KameTreeSerializer.hh */
/* Created by Enomoto Sanshiro on 9 April 2012. */
/* Last updated by Enomoto Sanshiro on 9 April 2012. */

#ifndef __KameTreeSerializer_hh__
#define __KameTreeSerializer_hh__

#include <string>
#include <deque>
#include "KameTree.hh"
#include "KameTreeWalker.hh"


namespace kame {


class TKameTreeSerializer: public TKameTreeHandler {
  public:
    TKameTreeSerializer(std::ostream& Output);
    virtual ~TKameTreeSerializer();
    virtual void Serialize(TKameTree& Tree, const std::string& Name = "");
  protected:
    std::ostream& _Output;
};


class TKameKtfTreeSerializer: public TKameTreeSerializer {
  public:
    TKameKtfTreeSerializer(std::ostream& Output);
    virtual ~TKameKtfTreeSerializer();
    virtual bool StartTree(TKameTree& Node);
    virtual bool StartNode(const std::string& Name, TKameTree& Node, TKameTree& AttributeList);
    virtual void EndNode(const std::string& Name, TKameTree& Node);
    virtual bool StartArray(const std::string& Name, TKameTree& Node);
  protected:
    char _HeaderChar;
    int _IndentStep, _Depth;
};


class TKameJsonTreeSerializer: public TKameTreeSerializer {
  public:
    TKameJsonTreeSerializer(std::ostream& Output);
    virtual ~TKameJsonTreeSerializer();
    virtual bool StartTree(TKameTree& Node);
  protected:
    virtual void ProcessNode(TKameTree& Tree, std::string NodeName, const std::string& Indent);
  protected:
    static void WriteJsonString(std::ostream& os, const std::string& Value);
};


class TKameXmlTreeSerializer: public TKameTreeSerializer {
  public:
    TKameXmlTreeSerializer(std::ostream& Output);
    virtual ~TKameXmlTreeSerializer();
    virtual bool StartTree(TKameTree& Node);
    virtual bool StartNode(const std::string& Name, TKameTree& Node, TKameTree& AttributeList);
    virtual void EndNode(const std::string& Name, TKameTree& Node);
    virtual bool StartArray(const std::string& Name, TKameTree& Node);
  protected:
    static void WriteXmlString(std::ostream& os, const std::string& Value);
  protected:
    std::deque<std::string> _IndentStack;
    std::string _NameForEmptyName;
};


class TKamePlistTreeSerializer: public TKameTreeSerializer {
  public:
    TKamePlistTreeSerializer(std::ostream& Output);
    virtual ~TKamePlistTreeSerializer();
    virtual bool StartTree(TKameTree& Node);
  protected:
    virtual void ProcessNode(TKameTree& Tree, std::string NodeName, const std::string& Indent);
  protected:
    static void WritePlistString(std::ostream& os, const std::string& Value);
};

}
#endif
