/* KameTreeWalker.cc */
/* Created by Enomoto Sanshiro on 9 April 2012. */
/* Last updated by Enomoto Sanshiro on 9 April 2012. */


#include <string>
#include "KameTree.hh"
#include "KameTreeWalker.hh"

using namespace std;
using namespace kame;


TKameTreeWalker::TKameTreeWalker(TKameTreeHandler* Handler)
{
    _Handler = Handler;
}

TKameTreeWalker::~TKameTreeWalker()
{
}

void TKameTreeWalker::Process(TKameTree& Tree, std::string RootNodeName)
{
    if (! _Handler) {
        return;
    }

    if (RootNodeName.empty()) {
        RootNodeName = Tree.NodeName();
    }

    if (_Handler->StartTree(Tree)) {
        ProcessNode(RootNodeName, Tree);
    }
    _Handler->EndTree(Tree);
}

void TKameTreeWalker::ProcessNode(const std::string& Name, TKameTree& Node)
{
    if (Node.IsArray()) {
        if (! _Handler->StartArray(Name, Node)) {
            return;
        }
    }

    for (unsigned i = 0; i < Node.Length(); i++) {
        TKameTree& Element = Node[i];
        TKameTree AttributeList;
        vector<string> KeyList;
        for (unsigned j = 0; j < Element.KeyList().size(); j++) {
            const string& Key = Element.KeyList()[j];
            if (Key.empty()) {
                // array of array; not supported in XML Tree model //
            }
            else if (Key[0] == '@') {
                AttributeList[Key.substr(1)] = Element[Key];
            }
            else {
                KeyList.push_back(Key);
            }
        }
        if (_Handler->StartNode(Name, Element, AttributeList)) {
            for (unsigned j = 0; j < KeyList.size(); j++) {
                TKameTree& ChildNode = Element[KeyList[j]];
                if (! ChildNode.IsEmpty()) {
                    ProcessNode(KeyList[j], ChildNode);
                }
            }
            _Handler->EndNode(Name, Element);
        }
    }

    if (Node.IsArray()) {
        _Handler->EndArray(Name, Node);
    }
}
