/* KameVariant.hh */
/* Created by Enomoto Sanshiro on 29 June 2009. */
/* Last updated by Enomoto Sanshiro on 29 June 2009. */


#include <string>
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <cerrno>
#include "KameVariant.hh"


using namespace std;
using namespace kame;


bool TKameVariant::AsBool(void) const
{
    if (_Type == Type_Void) {
#if 1
        throw TKameException("conversion from undefined to bool");
#else
        return false;
#endif
    }
    else if (_Type == Type_Bool) {
        return _Primitive._BoolValue;
    }
    if (_Type == Type_Long) {
	return _Primitive._LongValue != 0;
    }
    else if (_Type == Type_Double) {
	return _Primitive._DoubleValue != 0;
    }
    else if (_Type == Type_String) {
#if 1
        if (*_Primitive._StringValue == "true") {
            return true;
        }
        else if (*_Primitive._StringValue == "false") {
            return false;
        }
#else
        return ! _Primitive._StringValue->empty();
#endif
    }
    else if (_Type == Type_Unknown) {
        return _Primitive._UnknownValue != 0;
    }
    
    throw TKameException(
	"bad type to convert to bool: \"" + AsString() + "\""
    );
}

long TKameVariant::AsLong(void) const
{
    if (_Type == Type_Void) {
        throw TKameException("conversion from undefined to integer");
    }
    else if (_Type == Type_Bool) {
        return _Primitive._BoolValue ? 1 : 0;
    }
    else if (_Type == Type_Long) {
	return _Primitive._LongValue;
    }
    else if (_Type == Type_Double) {
	return (long) _Primitive._DoubleValue;
    }
    else if (_Type == Type_String) {
	long Value;
	const char* Start = _Primitive._StringValue->c_str();
	char* End;
	errno = 0;
	if (strncmp(Start, "0x", 2) == 0) {
	    Value = strtol(Start, &End, 0);
	}
	else {
	    Value = strtol(Start, &End, 10);
	}
	if (((Value == 0) && (errno != 0)) || (*End != '\0')) {
	    throw TKameException(
		"bad string to convert to int: \"" + 
		*_Primitive._StringValue + "\""
	    );
	}
	return Value;
    }
    
    throw TKameException(
	"bad type to convert to int: \"" + AsString() + "\""
    );
}

double TKameVariant::AsDouble(void) const
{
    if (_Type == Type_Void) {
        throw TKameException("conversion from undefined to double");
    }
    else if (_Type == Type_Bool) {
        return _Primitive._BoolValue ? 1 : 0;
    }
    else if (_Type == Type_Long) {
	return (double) _Primitive._LongValue;
    }
    else if (_Type == Type_Double) {
	return _Primitive._DoubleValue;
    }
    else if (_Type == Type_String) {
	double Value;
	const char* Start = _Primitive._StringValue->c_str();
	char* End;
	errno = 0;
	Value = strtod(Start, &End);
	if (((Value == 0) && (errno != 0)) || (*End != '\0')) {
	    throw TKameException(
		"bad string to convert to double: \"" + 
		*_Primitive._StringValue + "\""
	    );
	}
	return Value;
    }

    throw TKameException(
	"bad type to convert to int: \"" + AsString() + "\""
    );
}

std::string TKameVariant::AsString(void) const
{
    if (_Type == Type_String) {
	return *_Primitive._StringValue;
    }
    if (_Type == Type_Void) {
	return "";
    }

    std::ostringstream os;
    if (_Type == Type_Bool) {
	os << (_Primitive._BoolValue ? "true" : "false");
    }
    else if (_Type == Type_Long) {
	os << _Primitive._LongValue;
    }
    else if (_Type == Type_Double) {
	os << _Primitive._DoubleValue;
    }
    else {
	os << "Unknown@" << this;
    }

    return os.str();
}

TKameUnknown& TKameVariant::AsUnknown(void)
{
    if (_Type == Type_Void) {
        throw TKameException("conversion from undefined to unknown");
    }
    else if (_Type != Type_Unknown) {
	throw TKameException(
	    "bad type to convert to 'unknown': " + AsString()
	);
    }
    
    return *_Primitive._UnknownValue;
}

const TKameUnknown& TKameVariant::AsUnknown(void) const
{
    if (_Type == Type_Void) {
        throw TKameException("conversion from undefined to unknown");
    }
    else if (_Type != Type_Unknown) {
	throw TKameException(
	    "bad type to convert to 'unknown': " + AsString()
	);
    }
    
    return *_Primitive._UnknownValue;
}
