/* KiscNtuple.hh */
/* Created by Enomoto Sanshiro on 10 July 2009. */
/* Last updated by Enomoto Sanshiro on 10 July 2009. */


#ifndef __KiscNtuple_hh_
#define __KiscNtuple_hh_

#include <string>
#include <vector>
#include "ParaParser.hh"
#include "KameNtuple.hh"
#include "KameGraph.hh"
#include "KameHistogram.hh"
#include "KameHistogram2d.hh"
#include "KamePlainTextRepository.hh"


class TKiscTableFormatHeaderProcessor;


class TKameUnknownParaValue: public kame::TKameUnknown {
  public:
    TKameUnknownParaValue(const TParaValue& Value);
    TKameUnknownParaValue(const TKameUnknownParaValue& Value);
    virtual ~TKameUnknownParaValue(void);
    virtual TKameUnknown* Clone(void) const;
    TParaValue& ParaValue(void);
  protected:
    TParaValue _ParaValue;
};


class TKiscKameObject {
  public:
    TKiscKameObject(void);
    TKiscKameObject(const TKiscKameObject& Object);
    virtual ~TKiscKameObject();
    virtual TKiscKameObject& operator=(const TKiscKameObject& Object);
    virtual void SetDelimiter(const std::string& Delimiter);
    virtual void SetQuote(const std::string& Quote);
    virtual void Save(std::string FileName, std::string ObjectName);
    virtual void Load(std::string FileName, std::string ObjectName);
    virtual void TokenizeLine(const std::string& Line, std::vector<std::string>& TokenList);
  protected:
    virtual void SaveTo(kame::TKameRepository* Repository, const std::string& Name) = 0;
    virtual void LoadFrom(kame::TKameRepository* Repository, const std::string& Name) = 0;
    virtual void OnLoad(void) {}
  protected:
    virtual void Construct(void);
    virtual void ProcessScript(const std::string& ScriptText);
  protected:
    std::string _Delimiter, _Quote;
    TKiscTableFormatHeaderProcessor* _HeaderProcessor;
    TParaParser* _Parser;
    TParaPackage* _Package;
    std::vector<TParaPackageEntry*> _EntryList;
    TParaExpressionParser* _ExpressionParser;
    TParaStatementParser* _StatementParser;
    TParaSymbolTable* _SymbolTable;
};


class TKiscTableFormatHeaderProcessor: public kame::TKameTableFormatHeaderProcessor {
  public:
    TKiscTableFormatHeaderProcessor(void);
    virtual ~TKiscTableFormatHeaderProcessor();
    virtual bool ProcessLine(const std::string& Line);
    virtual const std::string& ScriptText(void);
  protected:
    std::string _ScriptText;
};


class TKiscNtuple: public kame::TKameNtuple, virtual public TKiscKameObject {
  public:
    TKiscNtuple(void);
    TKiscNtuple(int NumberOfColumns);
    TKiscNtuple(const TKiscNtuple& Ntuple);
    virtual ~TKiscNtuple();
    virtual TKiscNtuple& operator=(const TKiscNtuple& Ntuple);
  protected:
    virtual void SaveTo(kame::TKameRepository* Repository, const std::string& Name);
    virtual void LoadFrom(kame::TKameRepository* Repository, const std::string& Name);
    virtual void OnLoad(void);
  protected:
    virtual void EvalElements(void);
};


class TKiscGraph: public kame::TKameGraph, virtual public TKiscKameObject {
  public:
    TKiscGraph(unsigned InitialListSize = 1024);
    TKiscGraph(const kame::TKameGraph& Graph);
    TKiscGraph(const TKiscGraph& Graph);
    virtual ~TKiscGraph();
    virtual TKiscGraph& operator=(const TKiscGraph& Graph);
  protected:
    virtual void SaveTo(kame::TKameRepository* Repository, const std::string& Name);
    virtual void LoadFrom(kame::TKameRepository* Repository, const std::string& Name);
};


class TKiscHistogram: public kame::TKameHistogram, virtual public TKiscKameObject {
  public:
    TKiscHistogram();
    TKiscHistogram(const kame::TKameHistogramScale& Scale);
    TKiscHistogram(const TKiscHistogram& Histogram);
    virtual ~TKiscHistogram();
    virtual TKiscHistogram& operator=(const TKiscHistogram& Histogram);
  protected:
    virtual void SaveTo(kame::TKameRepository* Repository, const std::string& Name);
    virtual void LoadFrom(kame::TKameRepository* Repository, const std::string& Name);
};


class TKiscHistogram2d: public kame::TKameHistogram2d, virtual public TKiscKameObject {
  public:
    TKiscHistogram2d(void);
    TKiscHistogram2d(const kame::TKameHistogramScale& XScale, const kame::TKameHistogramScale& YScale);
    TKiscHistogram2d(const TKiscHistogram2d& Histogram2d);
    virtual ~TKiscHistogram2d();
    virtual TKiscHistogram2d& operator=(const TKiscHistogram2d& Histogram2d);
  protected:
    virtual void SaveTo(kame::TKameRepository* Repository, const std::string& Name);
    virtual void LoadFrom(kame::TKameRepository* Repository, const std::string& Name);
};


#endif
