/* KumaWavelet.hh */
/* Created by Sanshiro Enomoto on 29 December 2010 */


// TKumaXXXWavelet::Transform() は高速ウェーブレット変換を行なう．
// DataList をウェーブレット変換し，結果を同じ配列にもどす．
// DataLength は 2 の整数乗 (n = 2**m) でなければならない．
//
// InverseTransform() は逆変換を行う
// 
// 結果の格納は，スケールの大きい（遅い）要素から小さい（速い）要素の順に，
// 各スケールでは時間の順になっている


#ifndef __KumaWaveletTransform_hh__
#define __KumaWaveletTransform_hh__

#include <vector>
#include "KumaDefs.hh"


namespace kuma {


class TKumaWavelet {
  public:
    TKumaWavelet(void);
    virtual ~TKumaWavelet();
    virtual void Transform(double* DataList, int DataLength);
    virtual void InverseTransform(double* DataList, int DataLength);
  protected:
    void SetCoefficients(const double* CoefficientList, int Length);
  protected:
    int _FilterLength;
    std::vector<double> _SmoothFilter, _DetailFilter;
};


class TKumaHaarWavelet: public TKumaWavelet {
  public:
    TKumaHaarWavelet(void);
    virtual ~TKumaHaarWavelet();
};

class TKumaDaubechiesWavelet: public TKumaWavelet {
  public:
    TKumaDaubechiesWavelet(int NumberOfCoefficients);
    virtual ~TKumaDaubechiesWavelet();
};


}

#endif
