/* MushFileSystem.hh */
/* Created by Enomoto Sanshiro on 17 July 2001. */
/* Last updated by Enomoto Sanshiro on 15 October 2001. */


#ifndef __MushFileSystem_hh__
#define __MushFileSystem_hh__


#include <string>
#include "MushDefs.hh"


class TMushFileSystem {
  public:
    TMushFileSystem(const std::string& Path = "");
    virtual ~TMushFileSystem();
    static void ChangeDirectory(const std::string& Path);
    static std::string CurrentDirectory(void);
    static int MakeDirectory(const std::string& Name, int AccessMode = 0755);
    static unsigned SetUMask(unsigned UMask);
    static void Unlink(const std::string& FileName);
    virtual long FileSystemType(void);
    virtual long BlockSize(void);
    virtual long TotalNumberOfBlocks(void);
    virtual long NumberOfFreeBlocks(void);
    virtual long NumberOfAvailableBlocks(void);
    virtual long TotalNumberOfNodes(void);
    virtual long NumberOfFreeNodes(void);
  protected:
    std::string _Path;
    long _BlockSize;
};


class TMushFileAttribute {
  public:
    TMushFileAttribute(const std::string& FilePathName);
    virtual ~TMushFileAttribute();
    virtual long Size(void);
    virtual long AccessMode(void);
    virtual long OwnerUserId(void);
    virtual long OwnerGroupId(void);
    virtual long LastAccessTime(void);
    virtual long LastModificationTime(void);
    virtual long LastChangeTime(void);
    virtual unsigned long CheckSum(void);
    virtual bool IsFile(void);
    virtual bool IsDirectory(void);
    virtual bool IsSymbolicLink(void);
    virtual bool Exists(void);
    virtual bool IsReadable(void);
    virtual bool IsWritable(void);
    virtual bool IsExecutable(void);
    virtual std::string PathName(void);
    virtual std::string FileName(void);
    virtual std::string FileRootName(void);
    virtual std::string Extension(void);
  public:
    bool IsReadableFile(void) { return IsReadable() && IsFile(); }
    bool IsWritableFile(void) { return IsReadable() && IsFile(); }
    bool IsExecutableFile(void) { return IsReadable() && IsFile(); }
  protected:
    std::string _FilePathName;
};


#endif
