/* ParaToken.hh */
/* Created by Enomoto Sanshiro on 24 July 1999. */
/* Last updated by Enomoto Sanshiro on 12 July 2008. */


#ifndef __ParaToken_hh__
#define __ParaToken_hh__


#include <string>
#include <complex>
#include "ParaDefs.hh"


class TParaToken {
  public:
    enum TTokenType {
        ttKeyword,
        ttIdentifier,
        ttBool,
        ttInteger, 
        ttFloating, 
        ttComplex, 
        ttSeparator,
        ttOperator,
        ttQuote, 
	ttComment,
	ttWhiteSpace,
        ttEmpty, 
        ttUnknown
    };
    enum TNumberSuffixFlag {
	nsUnsigned = 0x01,
	nsLong = 0x02,
	nsImaginary = 0x04
    };
  public:
    TParaToken(void);
    TParaToken(const std::string& TokenString, TTokenType Type, unsigned NumberSuffixFlags, long LineNumber);
    TParaToken(const TParaToken& Token);
    virtual ~TParaToken();
    virtual TParaToken& operator=(const TParaToken& Token);
    virtual bool IsKeyword(void) const;
    virtual bool IsIdentifier(void) const;
    virtual bool IsBool(void) const;
    virtual bool IsInteger(void) const;
    virtual bool IsFloating(void) const;
    virtual bool IsComplex(void) const;
    virtual bool IsSeparator(void) const;
    virtual bool IsOperator(void) const;
    virtual bool IsQuote(void) const;
    virtual bool IsEmpty(void) const;
    virtual bool IsComment(void) const;
    virtual bool IsWhiteSpace(void) const;
    virtual bool Is(const std::string& String) const;
    virtual bool IsNot(const std::string& String) const;
    virtual std::string AsString(void) const;
    virtual bool AsBool(void) const;
    virtual long AsLong(void) const;
    virtual double AsDouble(void) const;
    virtual std::complex<double> AsComplex(void) const;
    virtual TParaToken& RemoveQuotation(char Quoter = '\0');
    virtual TParaToken& MustBe(const std::string& ExpectedString);
    virtual TParaToken& MustBe(TTokenType ExpectedTokenType);
    virtual void ThrowUnexpected(const std::string& Expected = "");
    virtual std::string Position(void) const;
  protected:
    std::string _TokenString;
    TTokenType _Type;
    unsigned _NumberSuffixFlags;
    long _LineNumber;
};


#endif
