/* RoomServiceRequester.hh */
/* Created by Enomoto Sanshiro on 7 July 2001. */
/* Last updated by Enomoto Sanshiro on 29 March 2002. */


#ifndef __RoomServiceRequester_hh__
#define __RoomServiceRequester_hh__


#include "RoomConfig.hh"
#include "RoomDefs.hh"


class TRoomServiceRequester {
  public:
    TRoomServiceRequester(void);
    virtual ~TRoomServiceRequester();

    // Basic Interface //
    virtual bool WaitForServiceRequest(int TimeOut_sec);
    virtual int RequestingServiceNumber(void);

    // Polling Interface //
    virtual void EnableServiceRequest(void);
    virtual void DisableServiceRequest(void);
    virtual void ClearServiceRequest(void);
    virtual bool IsRequestingService(void);

    // Signal on Service Request //
    virtual bool IsSignalOnServiceRequestAvailable(void);
    virtual void EnableSignalOnServiceRequest(int SignalId);
    virtual void DisableSignalOnServiceRequest(void);
};


class TRoomServiceRequestHandler {
  public:
    TRoomServiceRequestHandler(void);
    virtual ~TRoomServiceRequestHandler();
    virtual void SetPollingInterval(int Interval_usec);
    virtual void ForcePolling(void);
    virtual void RegisterRequester(TRoomServiceRequester* Requester, int RequesterId);
    virtual void Initialize(void);
    virtual void Finalize(void);
    virtual bool WaitForServiceRequest(int& RequesterId, int& RequestNumber, int TimeOut_sec = 1);
    virtual void ClearServiceRequest(void);
  protected:
    virtual int LookForRequestSignal(int TimeOut_sec);
    virtual int PollRequest(int TimeOut_sec);
  protected:
    int _NumberOfRequesters;
    int _RequesterListCapacity;
    TRoomServiceRequester** _RequesterList;
    int* _RequesterIdList;
    bool _IsSignalAvailable;
    int _PollingInterval_usec;
    TRoomServiceRequester* _RequestingRequester;
    bool _IsPollingForced;
  private:
    void StartSignalHandling(void);
    void StopSignalHandling(void);
    int WaitForSignal(int TimeOut_sec);
    static void SignalHandler(int SignalId);
    static const int _SignalId;
    static int _SignalCount;
    static void (*_OldSignalHandler)(int);
};


#endif
