/* bridge-Toyo_CCNET.hh */
/* Created by Enomoto Sanshiro on 22 July 2009. */
/* Last updated by Enomoto Sanshiro on 15 January 2010. */


#ifndef __bridge_Toyo_CCNET_hh__
#define __bridge_Toyo_CCNET_hh__


#include "RoomNetworkBridge.hh"


class TMiscDevice_Toyo_CCNET {
  public:
    TMiscDevice_Toyo_CCNET(void);
    virtual ~TMiscDevice_Toyo_CCNET();
    void CCNET_Open(TRoomNetworkConnection* NetworkConnection);
    void CCNET_Close(void);
    void CCNET_Reset(void);
    void CCNET_ClearFifo(void);
    void CCNET_EnableLam(int LamPattern = 0x00ffffff);
    void CCNET_DisableLam(void);
    int CCNET_WaitLam(int Timeout_Ticks);
    void CCNET_EnableTrigger(void);
    void CCNET_DisableTrigger(void);
    int CCNET_WaitTrigger(int Timeout_Ticks);
    void CCNET_Put(int Data, int Command);
    void CCNET_Get(int& Data, int& Reply);
  public:
    // Pipeline Execution //
    int* CCNET_CreateBuffer(void);
    void CCNET_DestroyBuffer(int* Buffer);
    int CCNET_ExecuteFrame(int Command, const int* CommandBuffer, int* ReplyBuffer);
    int CCNET_ExecutePio(const int* CommandBuffer, int* ReplyBuffer);
    int CCNET_ExecuteDma(const int* CommandBuffer, int* ReplyBuffer);
    int CCNET_ExecuteDmaSequence(const int* CommandBuffer, int* ReplyBuffer);
    int CCNET_Execute(const int* CommandBuffer, int* ReplyBuffer);
  public:
    // Frame Formatting and Scanning //
    void CCNET_FillCamacCommandTo(int Command[2], int N, int A, int F, int Data, int FrameFlag = 0);
    void CCNET_FillDaqCommandTo(int Command[2], int Function, int FrameFlag = 0);
    void CCNET_ExtractCamacReplyFrom(int Reply[2], int& N, int& A, int& F, int& Data, int&Q, int&X);
    void CCNET_ExtractDaqReplyFrom(int Reply[2], int& Data, int& Status);
    bool CCNET_IsCamacReply(int Reply[2]);
  public:
    // Command Buffer Building //
    void CCNET_InitializeCommandBuffer(int* CommandBuffer);
    void CCNET_CompleteCommandBuffer(int* CommandBuffer);
    void CCNET_LoadCamacCommand(int* CommandBuffer, int N, int A, int F, int Data);
    void CCNET_LoadDaqCommand(int* CommandBuffer, int Command, int Data);
  public:
    // Common Construction //
    void CCNET_ExecuteSingleCamac(int N, int A, int F, int& Data, int& Q, int& X);
    void CCNET_ExecuteSingleDaq(int Function, int& Data, int& Status);
  private:
    int* _ReceiveBuffer;
    int* _SendBuffer;
    TRoomNetworkConnection* _Connection;
  public:
    enum TParameters {
	_WordSize = 4,  // 32bit
	_MaxBufferSize = 128 * 1024,
	_MaxFrameLength = _MaxBufferSize/8-2,
	_PacketHeaderSize = 2 * _WordSize,
	_BufferSize = 64 * _WordSize,  // > sizeof(struct pccreg)
	_PacketSize = _PacketHeaderSize + _BufferSize,
	_NumberOfParameters
    };
    enum TPacketOffset {
	Offset_TotalSize = 0,
	Offset_Command = 1,
	Offset_DataStart = 2,      // Data Packet
	Offset_FrameCapacity = 2,  // CMD/RCV Packet
	Offset_NumberOfFrames = 3, // CMD/RCV Packet
	Offset_FrameStart = 4,     // CMD/RCV Packet
	_OffsetTableSize
    };
    enum TCommand {
	Command_Open = 1,
	Command_Close = 2,
	Command_Reset = 4,
	Command_ClearFifo = 5,
	Command_EnableLam = 10,
	Command_WaitLam = 11,
	Command_DisableLam = 12,
	Command_EnableTrigger = 13,
	Command_WaitTrigger = 14,
	Command_DisableTrigger = 15,
	Command_Get = 101,
	Command_Put = 102,
	Command_GetInterrupt = 103,
	Command_ExecutePio = 104,
	Command_ExecuteDma = 105,
	Command_ExecuteDmaSequence = 106,
	Command_Execute = 107,
	_NumberOfCommands
    };
    enum TFrameFlag {
	FrameFlag_Normal = 0x80,
	FrameFlag_Start = 0xC0,
	FrameFlag_End = 0xA0,
	FrameFlag_Single = 0xE0,
	FrameFlag_Camac = 0x00,
	FrameFlag_Daq = 0x10,
	_NumberOfFrameFlags
    };
};


class TNetworkCamacBridge_Toyo_CCNET: public TRoomNetworkCamacBridge, protected TMiscDevice_Toyo_CCNET {
  public:
    TNetworkCamacBridge_Toyo_CCNET(void);
    virtual ~TNetworkCamacBridge_Toyo_CCNET();
    virtual TRoomNetworkCamacBridge* CloneBridge(void);
  public:
    // Camac Controller Interface //
    virtual bool ProbeController(void);
    virtual int InitializeController(int InitialState = 0);
    virtual int ClearController(void);
    virtual void Open(void);
    virtual void Close(void);
    virtual int Transact(int StationNumber, int Function, int Address, int &Data, int &Q, int &X);
    virtual int SetInhibit(void);
    virtual int ReleaseInhibit(void);
    virtual int EnableInterruptNotification(void);
    virtual int DisableInterruptNotification(void);
    virtual int ReadLam(int LamMask);
    virtual int WaitLam(int TimeOut_sec);
  public:
    // Network Module Interface //
    virtual int AddressBitLength(void);
    virtual int DataBitLength(void);
    virtual int Initialize(int InitialState = 0);
    virtual int Finalize(int FinalState = 0);
    virtual int Clear(int Address = -1);
    virtual int BlockRead(int Address, void *Data, int MaxSize);
    virtual int NextDataBlockSize(int Address = -1);
    virtual void ClearServiceRequest(void);
    virtual bool WaitForServiceRequest(int TimeOut_sec);
    virtual bool IsRequestingService(void);
    virtual int MiscControlIdOf(const std::string& CommandName);
    virtual int MiscControl(int ControlId, int* ArgumentList = 0, int NumberOfArguments = 0);
  public:
  public:
    enum TControlId {
	ControlId_ExecuteSingleCamac,
	ControlId_ExecuteSingleDaq,
	ControlId_InitializeCommandBuffer,
	ControlId_LoadCamacCommand,
	ControlId_LoadDaqCommand,
	ControlId_Execute,
	ControlId_ExecutePio,
	ControlId_ExecuteDma,
	ControlId_ExecuteDmaSequence,
	_NumberOfControls
    };
  private:
    int* _CommandBuffer;
    int* _ReplyBuffer;
    int _ReplySize;
};


#endif
