/* controller-CAEN_VMELib.cc */
/* Created by Enomoto Sanshiro on 9 September 2014. */
/* Updated by Enomoto Sanshiro on 6 April 2015. */

//// No Memory Mapping ////
//// No Interrupt Handling (yet) ////

#include <sys/types.h>
#include "RoomDeviceFactory.hh"
#include "controller-CAEN_VMELib.hh"

#define LINUX
#include <CAENVMElib.h>

using namespace std;


static TRoomVmeControllerCreator Creator0(
    "CAEN_VMELib_V1718", 
    new TVmeController_CAEN_VMELib(TVmeController_CAEN_VMELib::btV1718)
);
static TRoomVmeControllerCreator Creator1(
    "CAEN_VMELib_V2718", 
    new TVmeController_CAEN_VMELib(TVmeController_CAEN_VMELib::btV2718)
);

static const CVAddressModifier AddressModifierTable[] = {
    /* this must be consistent with TRoomVmeTarnsferMode */
    /* (declared in RoomVmeAccess.hh) */
    cvA16_U, cvA16_U,
    cvA24_U_DATA, cvA24_U_DATA,
    cvA32_U_DATA, cvA32_U_DATA
};
static const CVAddressModifier DmaAddressModifierTable[] = {
    /* this must be consistent with TRoomVmeTarnsferMode */
    /* (declared in RoomVmeAccess.hh) */
    cvA16_U, cvA16_U,
    cvA24_U_BLT, cvA24_U_BLT,
    cvA32_U_BLT, cvA32_U_BLT
};
static const CVDataWidth DataWidthTable[] = {
    /* this must be consistent with TRoomVmeTarnsferMode */
    /* (declared in RoomVmeAccess.hh) */
    cvD16, cvD32,
    cvD16, cvD32,
    cvD16, cvD32
};


struct TDeviceSpecificData {
    off_t SeekPosition;
};

static string ErrorMessageOf(CAENVME_API ReturnCode)
{
    string Message;
    switch (ReturnCode) {
      case cvBusError:
        Message = "VME bus error during the cycle";
        break;
      case cvCommError:
        Message = "Communication error";
        break;
      case cvGenericError:
        Message = "Unspecified error";
        break;
      case cvInvalidParam:
        Message = "Invalid parameter";
        break;
      case cvTimeoutError:
        Message = "Timeout";
        break;
      default:
        Message = "Unknown error";
        break;
    }

    return Message;
}


TVmeController_CAEN_VMELib::TVmeController_CAEN_VMELib(int BoardType)
{
    _IsOpened = false;
    _Handle = -1;
    _BoardType = (TBoardType) BoardType;
}

TVmeController_CAEN_VMELib::~TVmeController_CAEN_VMELib()
{
    if (_IsOpened) {
        CAENVME_End(_Handle);
    }
}

TRoomVmeController* TVmeController_CAEN_VMELib::CloneController(void)
{
    return new TVmeController_CAEN_VMELib(_BoardType);
}

const char* TVmeController_CAEN_VMELib::DeviceFileName(TRoomVmeTransferMode TransferMode)
{
    return 0;
}

const char* TVmeController_CAEN_VMELib::DmaDeviceFileName(TRoomVmeTransferMode TransferMode)
{
    return 0;
}

TRoomVmeAccessProperty* TVmeController_CAEN_VMELib::Open(TRoomVmeTransferMode TransferMode)
{
    if (TransferMode >= VmeTransferMode_Unknown) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::Open()", "bad vme transfer mode"
        );	
    }

    if (! _IsOpened) {
        CVBoardTypes BdType;
	if (_BoardType == btV2718) {
	  BdType = cvV2718;
	}
	else {
	  BdType = cvV1718;
	}
        short Link = 0;
        short MaxBdNum = 12;
        CAENVME_API Result;
        int32_t Handle;
        for (short BdNum = 0; BdNum < MaxBdNum; BdNum++) {
            Result = CAENVME_Init(BdType, Link, BdNum, &Handle);
            if (Result == cvSuccess) {
                cout << "CAEN VME detected at BdNum = " << BdNum << endl;
                break;
            }
        }
        if (Result != cvSuccess) {
            throw THardwareException(
                "TVmeController_CAEN_VMELib::Open()", 
                "unable to initialize the board: " + ErrorMessageOf(Result)
            );	
        }
        _Handle = Handle;
        _IsOpened = true;
    }

    TRoomVmeAccessProperty* Property = new TRoomVmeAccessProperty;
    Property->TransferMode = TransferMode;
    Property->VmeAddress = 0;
    Property->DeviceDescriptor = -1;
    Property->DmaDeviceDescriptor = -1;
    Property->MappedAddress = 0;
    Property->MapSize = 0;
    Property->MapOffset = 0;
    Property->InterruptNumber = 0;
    Property->InterruptVector = 0;

    Property->DeviceSpecificData = new TDeviceSpecificData();
    ((TDeviceSpecificData*) Property->DeviceSpecificData)->SeekPosition = 0;
    
    //... TODO: check this ...//
    Property->IsByteOrderCorrect = true;

    return Property;
}

void TVmeController_CAEN_VMELib::Close(TRoomVmeAccessProperty* Property)
{
    if (_IsOpened) {
        CAENVME_End(_Handle);
        delete (TDeviceSpecificData*) Property->DeviceSpecificData;
        _IsOpened = false;
    }
}

long TVmeController_CAEN_VMELib::Read(TRoomVmeAccessProperty* Property, void* Buffer, int Size)
{
    throw THardwareException(
        "TVmeController_CAEN_VMELib::Read()", "function not implemented"
    );

    return -1;
}

long TVmeController_CAEN_VMELib::Write(TRoomVmeAccessProperty* Property, const void* Buffer, int Size)
{
    throw THardwareException(
        "TVmeController_CAEN_VMELib::Write()", "function not implemented"
    );

    return -1;
}

off_t TVmeController_CAEN_VMELib::Seek(TRoomVmeAccessProperty* Property, off_t OffsetAddress)
{
    ((TDeviceSpecificData*) Property->DeviceSpecificData)->SeekPosition = OffsetAddress;
    return OffsetAddress;
}

caddr_t TVmeController_CAEN_VMELib::Map(TRoomVmeAccessProperty* Property, off_t VmeAddress, size_t MapSize, off_t MapOffset)
{
    // Mapped I/O is not available //
    // by returning 0 here, Put/Get methods below will be called instead //

    caddr_t MappedAddress = 0;

    Property->VmeAddress = VmeAddress;
    Property->MapSize = MapSize;
    Property->MapOffset = MapOffset;
    Property->MappedAddress = MappedAddress;

    return MappedAddress;
}

void TVmeController_CAEN_VMELib::Unmap(TRoomVmeAccessProperty* Property)
{
}

void TVmeController_CAEN_VMELib::PutWord(TRoomVmeAccessProperty* Property, off_t OffsetAddress, Word Value)
{
    off_t Address = Property->VmeAddress + OffsetAddress;
    CVAddressModifier AM = AddressModifierTable[Property->TransferMode];
    CVDataWidth DW = cvD16;

    CAENVME_API Result;
    Result = CAENVME_WriteCycle(_Handle, Address, &Value, AM, DW);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::PutWord()",
            "error on CAENVME_WriteCycle(): " + ErrorMessageOf(Result)
        );
    }
}

Word TVmeController_CAEN_VMELib::GetWord(TRoomVmeAccessProperty* Property, off_t OffsetAddress)
{
    off_t Address = Property->VmeAddress + OffsetAddress;
    CVAddressModifier AM = AddressModifierTable[Property->TransferMode];
    CVDataWidth DW = cvD16;

    Word Value;

    CAENVME_API Result;
    Result = CAENVME_ReadCycle(_Handle, Address, &Value, AM, DW);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::GetWord()",
            "error on CAENVME_ReadCycle(): " + ErrorMessageOf(Result)
        );
    }

    return Value;
}

void TVmeController_CAEN_VMELib::PutDoubleWord(TRoomVmeAccessProperty* Property, off_t OffsetAddress, DoubleWord Value)
{
    off_t Address = Property->VmeAddress + OffsetAddress;
    CVAddressModifier AM = AddressModifierTable[Property->TransferMode];
    CVDataWidth DW = cvD32;

    CAENVME_API Result;
    Result = CAENVME_WriteCycle(_Handle, Address, &Value, AM, DW);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::PutWord()",
            "error on CAENVME_WriteCycle(): " + ErrorMessageOf(Result)
        );
    }
}

DoubleWord TVmeController_CAEN_VMELib::GetDoubleWord(TRoomVmeAccessProperty* Property, off_t OffsetAddress)
{
    off_t Address = Property->VmeAddress + OffsetAddress;
    CVAddressModifier AM = AddressModifierTable[Property->TransferMode];
    CVDataWidth DW = cvD32;

    DoubleWord Value;

    CAENVME_API Result;
    Result = CAENVME_ReadCycle(_Handle, Address, &Value, AM, DW);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::GetDoubleWord()",
            "error on CAENVME_ReadCycle(): " + ErrorMessageOf(Result)
        );
    }

    return Value;
}

void TVmeController_CAEN_VMELib::DmaOpen(TRoomVmeAccessProperty* Property)
{
}

void TVmeController_CAEN_VMELib::DmaClose(TRoomVmeAccessProperty* Property)
{
}

long TVmeController_CAEN_VMELib::DmaRead(TRoomVmeAccessProperty* Property, void* Buffer, int Size)
{
    TDeviceSpecificData* Dev = (
        (TDeviceSpecificData*) Property->DeviceSpecificData
    );

    off_t VmeAddress = Dev->SeekPosition;
    CVAddressModifier AM = DmaAddressModifierTable[Property->TransferMode];
    CVDataWidth DW = DataWidthTable[Property->TransferMode];

    CAENVME_API Result;
    int Count;
    Result = CAENVME_BLTReadCycle(
        _Handle, VmeAddress, Buffer, Size, AM, DW, &Count
    );
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::DmaRead()",
            "error on CAENVME_BLTReadCycle(): " + ErrorMessageOf(Result)
        );
    }
    Dev->SeekPosition += Count;
    
    if (Size != Count) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::DmaRead()",
            "size error on CAENVME_BLTReadCycle()"
        );
    }

    return Size;
}

long TVmeController_CAEN_VMELib::DmaWrite(TRoomVmeAccessProperty* Property, const void* Buffer, int Size)
{
    throw THardwareException(
        "TVmeController_CAEN_VMELib::DmaWrite()", "function not implemented"
    );

    return -1;
}

off_t TVmeController_CAEN_VMELib::DmaSeek(TRoomVmeAccessProperty* Property, off_t OffsetAddress)
{
    return Seek(Property, OffsetAddress);
}

bool TVmeController_CAEN_VMELib::IsSignalOnInterruptAvailable(void)
{
    return false;
}

void TVmeController_CAEN_VMELib::RegisterInterruptNotification(TRoomVmeAccessProperty* Property, int SignalId)
{
    throw THardwareException(
        "TVmeController_CAEN_VMELib::RegisterInterruptNotification()", 
	"function is not available for controller: CAEN_VMELib"
    );
}

void TVmeController_CAEN_VMELib::UnregisterInterruptNotification(TRoomVmeAccessProperty* Property)
{
}

void TVmeController_CAEN_VMELib::EnableInterruptNotification(TRoomVmeAccessProperty* Property)
{
}

void TVmeController_CAEN_VMELib::DisableInterruptNotification(TRoomVmeAccessProperty* Property)
{
}

bool TVmeController_CAEN_VMELib::WaitForInterruptNotification(TRoomVmeAccessProperty* Property, unsigned TimeOut)
{
    throw THardwareException(
        "TVmeController_CAEN_VMELib::WaitForInterruptNotification()", 
	"function is not available for controller: CAEN_VMELib"
    );

    return false;
}

void TVmeController_CAEN_VMELib::SetInterruptNotificationAutodisable(TRoomVmeAccessProperty* Property)
{
    throw THardwareException(
        "TRoomVmeController::SetInterruptNotificationAutodisable()", 
	"function is not available for controller: CAEN_VMELib"
    );
}

int TVmeController_CAEN_VMELib::ReadInternalRegister(int Address)
{
    unsigned int Data;

    CAENVME_API Result;
    Result = CAENVME_ReadRegister(_Handle, (CVRegisters) Address, &Data);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::ReadInternalRegister()",
            "error on CAENVME_ReadRegister()"
        );
    }

    return Data;
}

void TVmeController_CAEN_VMELib::WriteInternalRegister(int Address, int Data)
{
    CAENVME_API Result;
    Result = CAENVME_WriteRegister(_Handle, (CVRegisters) Address, Data);
    if (Result != cvSuccess) {
	throw THardwareException(
            "TVmeController_CAEN_VMELib::ReadInternalRegister()",
            "error on CAENVME_ReadRegister()"
        );
    }
}
