/* module-ClearPulse_80057.cc */
/* Created by Enomoto Sanshiro on 6 April 2015. */

#include <sstream>
#include <unistd.h>
#include "RoomDeviceFactory.hh"
#include "RoomVmeAccess.hh"
#include "module-ClearPulse_80057.hh"

using namespace std;


static TRoomVmeModuleCreator Creator(
    "ClearPulse_80057", 
    new TVmeMisc_ClearPulse_80057()
);


TVmeMisc_ClearPulse_80057::TVmeMisc_ClearPulse_80057(void)
: TRoomVmeModule("VmeMisc", "ClearPulse_80057", (TRoomVmeTransferMode) TransferMode, (size_t) MapSize)
{
    _ModelNumber = 0;
    _FirmwareVersion = 0;

    _CurrentDpTriggerMode = 0;
    _CurrentPmtTriggerMode = 0;

    _EnabledPortFlags = 0x01;
    
    _IsSnigleReadDataReady = false;
}

TVmeMisc_ClearPulse_80057::~TVmeMisc_ClearPulse_80057()
{
}

TRoomVmeModule* TVmeMisc_ClearPulse_80057::Clone(void)
{
    return new TVmeMisc_ClearPulse_80057();
}

bool TVmeMisc_ClearPulse_80057::Probe(void)
{
    if (GetDoubleWord(regVendorId) != VendorId) {
        return false;
    }
    U32bit ModuleId = GetDoubleWord(regModuleId);
    _ModelNumber = ModuleId >> 8;
    _FirmwareVersion = ModuleId & 0x000f;

    if (_ModelNumber != KnownModelNumber) {
        return false;
    }
    if (_FirmwareVersion != KnownFirmwareVersion) {
        return false;
    }

    return true;
}

int TVmeMisc_ClearPulse_80057::NumberOfChannels(void)
{
    return (
        (_EnabledPortFlags & 0x8) ? 4 * _NumberOfChannels :
        (_EnabledPortFlags & 0x4) ? 3 * _NumberOfChannels :
        (_EnabledPortFlags & 0x2) ? 2 * _NumberOfChannels :
        (_EnabledPortFlags & 0x1) ? 1 * _NumberOfChannels :
        0
    );
}

int TVmeMisc_ClearPulse_80057::AddressBitLength(void)
{
    return _AddressBitLength;
}

int TVmeMisc_ClearPulse_80057::DataBitLength(void)
{
    return _DataBitLength;
}

bool TVmeMisc_ClearPulse_80057::ProcessCommand(const string& Command, const vector<string>& ArgumentList)
{
    if (Command == "setEnabledPortFlags") {
	if (ArgumentList.size() < 1) {
	    throw THardwareException(
		"ClearPulse_88057::setEnabledPortFlags(flags)", "too few argument"
	    );
	}
	if (! (istringstream(ArgumentList[0]) >> _EnabledPortFlags)) {
	    throw THardwareException(
		"ClearPulse_88057::setEnabledPortFlags(flags)", "invalid argument"
	    );
	}
    }
    else {
        return TRoomVmeModule::ProcessCommand(Command, ArgumentList);
    }

    return true;
}

int TVmeMisc_ClearPulse_80057::Initialize(int InitialState)
{
    _CurrentDpTriggerMode = 0;
    _CurrentPmtTriggerMode = 0;
    _IsSnigleReadDataReady = false;

    WritePmtCommand(_EnabledPortFlags, cmdReset);
    usleep(100);
    WritePmtCommand(_EnabledPortFlags, cmdReset);
    usleep(100);

    PutDoubleWord(regDpCommand, bitLedOn);
    PutDoubleWord(regDpCommand, bitPowerOn);
    sleep(1);

    PutDoubleWord(regDpPipeLine, 0);
    PutDoubleWord(regDpCommand, bitBusyOn);
    SetPmtTriggerMode(_EnabledPortFlags, 0x00);
    PutDoubleWord(regDpCommand, bitFifoReset);

    // no pedestal subtraction, no gain correction //
    for (unsigned i = 0; i < _NumberOfChannels; i++) {
        PutDoubleWord(regPedestalA + 4*i, 0);
        PutDoubleWord(regPedestalB + 4*i, 0);
        PutDoubleWord(regPedestalC + 4*i, 0);
        PutDoubleWord(regPedestalD + 4*i, 0);
    }
    for (unsigned i = 0; i < _NumberOfChannels; i++) {
        PutDoubleWord(regGainA + 4*i, 2048);
        PutDoubleWord(regGainB + 4*i, 2048);
        PutDoubleWord(regGainC + 4*i, 2048);
        PutDoubleWord(regGainD + 4*i, 2048);
    }

    PutDoubleWord(regPortMask, _EnabledPortFlags << 4); 
    sleep(1);

    PutDoubleWord(regTWidth, 63);
    PutDoubleWord(regVWidth, 240);
    PutDoubleWord(regBWidth, 255);
    PutDoubleWord(regHeadSizeA, 2);
    PutDoubleWord(regHeadSizeB, 2);
    PutDoubleWord(regHeadSizeC, 2);
    PutDoubleWord(regHeadSizeD, 2);
    PutDoubleWord(regSdsBlockSizeA, 64);
    PutDoubleWord(regSdsBlockSizeB, 64);
    PutDoubleWord(regSdsBlockSizeC, 64);
    PutDoubleWord(regSdsBlockSizeD, 64);
    PutDoubleWord(regHeaderIdA, 0xeb90);
    PutDoubleWord(regHeaderIdB, 0xeb91);
    PutDoubleWord(regHeaderIdC, 0xeb92);
    PutDoubleWord(regHeaderIdD, 0xeb93);

    WritePmtCommand(_EnabledPortFlags, cmdReset);
    WritePmtCommand(_EnabledPortFlags, cmdClear);
    WritePmtCommand(_EnabledPortFlags, cmdResetTime);
    PutDoubleWord(regDpCommand, bitFifoReset);

    SetPmtTriggerMode(_EnabledPortFlags, 0x00);
    ResetFecRegister(_EnabledPortFlags);
    SetPmtBusyExtension(_EnabledPortFlags, 5);
    SetPmtHoldTime(_EnabledPortFlags, 144);
    usleep(100);

    PutDoubleWord(regDpCommand, bitFifoReset);
    PutDoubleWord(regDpCommand, bitBusyOff);

    return 1;
}

int TVmeMisc_ClearPulse_80057::Finalize(int FinalState)
{
    PutDoubleWord(regDpCommand, bitPowerOff);
    PutDoubleWord(regDpCommand, bitLedOff);

    return 1;
}

int TVmeMisc_ClearPulse_80057::WriteRegister(int Address, int Data)
{
    PutDoubleWord(Address, Data);
    return 1;
}

int TVmeMisc_ClearPulse_80057::ReadRegister(int Address, int& Data)
{
    Data = GetDoubleWord(Address);
    return 1;
}

int TVmeMisc_ClearPulse_80057::MiscControlIdOf(const string& Command)
{
    int ControlId = -1;
    if (Command == "resetDpFifo") {
        ControlId = ControlId_ResetDpFifo;
    }
    else if (Command == "setDpBusyOn") {
        ControlId = ControlId_SetDpBusyOn;
    }
    else if (Command == "setDpBusyOff") {
        ControlId = ControlId_SetDpBusyOff;
    }
    else if (Command == "issueDpTrigger") {
        ControlId = ControlId_IssueDpTrigger;
    }
    else if (Command == "enableDpExtTrigIn") {
        ControlId = ControlId_EnableDpExtTrigIn;
    }
    else if (Command == "enableDpExtTrigOut") {
        ControlId = ControlId_EnableDpExtTrigOut;
    }
    else if (Command == "issuePmtTrigger") {
        ControlId = ControlId_IssuePmtTrigger;
    }
    else if (Command == "setPmtHoldTime") {
	ControlId = ControlId_SetPmtHoldTime;
    }
    else if (Command == "setPmtGainHigh") {
	ControlId = ControlId_SetPmtGainHigh;
    }
    else if (Command == "setPmtGainLow") {
	ControlId = ControlId_SetPmtGainLow;
    }
    else if (Command == "enablePmtExtTrigIn") {
        ControlId = ControlId_EnablePmtExtTrigIn;
    }
    else if (Command == "enablePmtExtTrigOut") {
        ControlId = ControlId_EnablePmtExtTrigOut;
    }
    else if (Command == "enablePmtAnodeTrigger") {
        ControlId = ControlId_EnablePmtAnodeTrigger;
    }
    else if (Command == "enablePmtDynodeTrigger") {
        ControlId = ControlId_EnablePmtDynodeTrigger;
    }
    else if (Command == "setPmtDynodeThreshold") {
        ControlId = ControlId_SetPmtDynodeThreshold;
    }
    else if (Command == "setPmtAnodeTriggerChannelEnable") {
        ControlId = ControlId_SetPmtAnodeTriggerChannelEnable;
    }
    else if (Command == "setPmtAnodeThreshold") {
        ControlId = ControlId_SetPmtAnodeThreshold;
    }

    else {
        throw THardwareException(
            "TVmeMisc_ClearPulse_80057::MiscControlIdOf()",
            "unknown command: " + Command
        );
    }

    return ControlId;
}

int TVmeMisc_ClearPulse_80057::MiscControl(int ControlId, int* ArgumentList, int NumberOfArguments)
{
    // DP (80057) Commands //
    if (ControlId == ControlId_ResetDpFifo) {
        ResetDpFifo();
    }
    else if (ControlId == ControlId_SetDpBusyOn) {
        SetDpBusyOn();
    }
    else if (ControlId == ControlId_SetDpBusyOff) {
        SetDpBusyOff();
    }
    else if (ControlId == ControlId_IssueDpTrigger) {
        IssueDpTrigger();
    }
    else if (ControlId == ControlId_EnableDpExtTrigIn) {
        SetDpTriggerMode(bitExtTrigIn);
    }
    else if (ControlId == ControlId_EnableDpExtTrigOut) {
        SetDpTriggerMode(bitExtTrigOut);
    }

    // PMT (80190) Commands //
    else if (ControlId == ControlId_IssuePmtTrigger) {
        IssuePmtTrigger(_EnabledPortFlags);
    }
    else if (ControlId == ControlId_SetPmtHoldTime) {
	if (NumberOfArguments < 1) {
	    throw THardwareException(
		"TVmeMisc_ClearPulse_80057::setPmtHoldTime(holdTime): ", 
		"too few argument[s]"
	    );
	}
        unsigned HoldTime = ArgumentList[0];
        SetPmtHoldTime(_EnabledPortFlags, HoldTime);
    }
    else if (ControlId == ControlId_SetPmtGainHigh) {
        SetPmtGain(_EnabledPortFlags, gainHigh);
    }
    else if (ControlId == ControlId_SetPmtGainLow) {
        SetPmtGain(_EnabledPortFlags, gainLow);
    }
    else if (ControlId == ControlId_EnablePmtExtTrigOut) {
        _CurrentPmtTriggerMode |= trigExtTrigOut;
        SetPmtTriggerMode(_EnabledPortFlags, _CurrentPmtTriggerMode);
    }
    else if (ControlId == ControlId_EnablePmtExtTrigIn) {
        _CurrentPmtTriggerMode |= trigExtTrigIn;
        SetPmtTriggerMode(_EnabledPortFlags, _CurrentPmtTriggerMode);
    }
    else if (ControlId == ControlId_EnablePmtAnodeTrigger) {
        _CurrentPmtTriggerMode |= trigAnodeTrig;
        SetPmtTriggerMode(_EnabledPortFlags, _CurrentPmtTriggerMode);
    }
    else if (ControlId == ControlId_EnablePmtDynodeTrigger) {
        _CurrentPmtTriggerMode |= trigDynodeTrig;
        SetPmtTriggerMode(_EnabledPortFlags, _CurrentPmtTriggerMode);
    }
    else if (ControlId == ControlId_SetPmtDynodeThreshold) {
	if (NumberOfArguments < 1) {
	    throw THardwareException(
		"TVmeMisc_ClearPulse_80057::setPmtDynodeThreshold(threshold): ", 
		"too few argument[s]"
	    );
	}
        unsigned Threshold = ArgumentList[0];
        SetPmtDynodeThreshold(_EnabledPortFlags, Threshold);
    }
    else if (ControlId == ControlId_SetPmtAnodeTriggerChannelEnable) {
	if (NumberOfArguments < 1) {
	    throw THardwareException(
		"TVmeMisc_ClearPulse_80057::setPmtAnodeTriggerChannelEnable(channel): ", 
		"too few argument[s]"
	    );
	}
        unsigned Channel = ArgumentList[0];
        SetPmtAnodeTriggerChannelEnable(Channel);
    }
    else if (ControlId == ControlId_SetPmtAnodeThreshold) {
	if (NumberOfArguments < 2) {
	    throw THardwareException(
		"TVmeMisc_ClearPulse_80057::setPmtAnodeThreshold(channel, threshold): ", 
		"too few argument[s]"
	    );
	}
        unsigned Channel = ArgumentList[0];
        unsigned Threshold = ArgumentList[1];
        SetPmtAnodeThreshold(Channel, Threshold);
    }

    else {
        return 0;
    }

    return 1;
}

bool TVmeMisc_ClearPulse_80057::WaitData(unsigned TimeOut_sec)
{
    return TRoomModule::WaitData(TimeOut_sec);
}

bool TVmeMisc_ClearPulse_80057::HasData(int Address)
{
    return (GetDoubleWord(regStatus) & 0x0f) != 0;
}

int TVmeMisc_ClearPulse_80057::NextDataBlockSize(int Address)
{
    return _DataBlockSize;
}

int TVmeMisc_ClearPulse_80057::Read(int Address, int &Data)
{
     static const int AnodeChannelOf[/*ReadoutChannel*/] = {
         0,  1,  6,  7,  8,  9, 14, 15,
        16, 17, 22, 23, 24, 25, 30, 31,
        32, 33, 38, 39, 40, 41, 46, 47,
        48, 49, 54, 55, 56, 57, 62, 63,
         2,  3,  4,  5, 10, 11, 12, 13,
        18, 19, 20, 21, 26, 27, 28, 29,
        34, 35, 36, 37, 42, 43, 44, 45,
        50, 51, 52, 53, 58, 59, 60, 61
    };
        
    if ((Address % 64) == 0) {
        static const int BlockReadLength = _NumberOfChannels/2+1;
	static U32bit BlockReadBuffer[BlockReadLength];

        if (this->BlockRead(Address / 64, BlockReadBuffer, 4*BlockReadLength) <= 0) {
            _IsSnigleReadDataReady = false;
            return 0;
        }
        _IsSnigleReadDataReady = true;

        // EventCount = BlockReadBuffer[0] & 0x0000ffff;
        // EventTime = BlockReadBuffer[1] & 0x1fffffff;

	int ReadoutChannel = 0;
	for (unsigned i = 0; i < _NumberOfChannels/2; i++) {
  	    U32bit DataWord = BlockReadBuffer[i+2];
	    _SingleReadBuffer[AnodeChannelOf[ReadoutChannel++]] = (
	        (DataWord >> 16) & 0x1fff
	    );
	    _SingleReadBuffer[AnodeChannelOf[ReadoutChannel++]] = (
	        (DataWord >> 0) & 0x1fff
	    );
	}

	for (unsigned i = 0; i < _NumberOfChannels; i++) {
	    if (_SingleReadBuffer[i] & 0x1000) {
  	        _SingleReadBuffer[i] = 0;  // underflow
	    }
	}
    }
    if (! _IsSnigleReadDataReady) {
        return 0;
    }

    Data = _SingleReadBuffer[Address % 64];

    return 1;
}

int TVmeMisc_ClearPulse_80057::BlockRead(int Address, void* Data, int MaxSize)
{
    if ((Address < 0) || (Address >= 4)) {
        return 0;
    }

    static const unsigned DataAddressList[] = {
        regDataA, regDataB, regDataC, regDataD
    };
    unsigned DataAddress = DataAddressList[Address];

    U32bit DataValue;
    unsigned DataIndex = 0, LoopCount = 0;
    static const int LoopMax = 100;
    while (LoopCount < LoopMax) {
        DataValue = GetDoubleWord(DataAddress);
	if (DataValue == DataValue_Empty) {
	    LoopCount++;
	    usleep(1);
	    continue;
	}
        if (DataIndex == 0) {
            if ((DataValue & DataHeadMask) == DataValue_Head) {
                ((U32bit*)Data)[DataIndex++] = DataValue;
                // EventCount = DataValue & 0x0000ffff;
            }
            else {
                cerr << "ClearPulse 80057 framing error" << endl;
            }
            continue;
	}
        if (DataIndex == 1) {
            ;
            // EventTime = DataValue & 0x1fffffff;
        }
	((U32bit*)Data)[DataIndex++] = DataValue;
	if (DataIndex >= _NumberOfChannels/2 + 2) {  // two header words
	    break;
	}
    }

    return DataIndex * sizeof(U32bit);
}


//// 80057 - 80190 access methods ////

int TVmeMisc_ClearPulse_80057::WritePmtCommand(int PortFlags, unsigned Command)
{
    unsigned Address = 0;
    if (PortFlags & 0x0001) Address |= regCommandA;
    if (PortFlags & 0x0002) Address |= regCommandB;
    if (PortFlags & 0x0004) Address |= regCommandC;
    if (PortFlags & 0x0008) Address |= regCommandD;

    while (GetDoubleWord(regStatus) & bitCommandBusy) {
        usleep(1);
    }

    PutDoubleWord(Address, Command);

    return 1;
}

int TVmeMisc_ClearPulse_80057::WriteFecRegister(int PortFlags, const char* RegisterBits)
{
    unsigned Command = cmdWriteRegister;
    Command <<= 15; Command |= _FecRegisterLength - 1;
    WritePmtCommand(PortFlags, Command);

    int BitIndex = 32, Word = 0;
    for (unsigned i = 0; i < _FecRegisterLength; i++) {
        BitIndex--;
        if (RegisterBits[i]) {
            Word |= 0x01 << BitIndex;
	}
	if (BitIndex == 0) {
	    WritePmtCommand(PortFlags, Word);
	    BitIndex = 32;
	    Word = 0;
	}
    }
    if (BitIndex) {
        WritePmtCommand(PortFlags, Word);
    }
    WritePmtCommand(PortFlags, 0);

    return 1;
}

int TVmeMisc_ClearPulse_80057::ReadFecRegister(int PortFlags, unsigned int* Data)
{
    for (int i = 0; i < 9; i++) {
        WritePmtCommand(PortFlags, cmdTrigger);
    }

    unsigned DataIndex = 0;
    for (unsigned Port = 0; Port < _NumberOfPorts; Port++) {
        if ((PortFlags & (0x001 << Port)) == 0) {
            continue;
        }
        static const unsigned DataAddressList[] = {
            regDataA, regDataB, regDataC, regDataD
        };
        unsigned DataAddress = DataAddressList[Port];

        U32bit DataValue;
        unsigned LoopCount = 0;
        static const int LoopMax = 100;
        while (LoopCount < LoopMax) {
            DataValue = GetDoubleWord(DataAddress);
            if (DataValue == DataValue_Empty) {
                LoopCount++;
                usleep(1);
                continue;
            }
            if ((DataValue & DataHeadMask) == DataValue_Head) {
                LoopCount = 0;
                while (LoopCount < LoopMax) {
                    DataValue = GetDoubleWord(DataAddress);
                    if (DataValue != DataValue_Empty) {
                        break;
                    }
                    LoopCount++;
                    usleep(1);
                }
            }
            if (((DataValue >> 24) & 0x00ff) != (0x00e0 | DataIndex)) {
                continue;
            }
            if (Data) {
                Data[DataIndex] = DataValue & 0x00ffffff;
            }
            DataIndex++;
            if (DataIndex >= 8) {
                break;
            }
        }
    }

    return 1;
}

int TVmeMisc_ClearPulse_80057::ResetFecRegister(int PortFlags)
{
    // Reset DAC
    for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
        _FecRegister.FillFpga(Fec, 1);
        _FecRegister.FillDacControl(Fec, TFecRegister::cmdReset, 0);
    }
    WriteFecRegister(PortFlags, _FecRegister.Bits);
    usleep(1350);
    ReadFecRegister(PortFlags, 0);

    // Turn on the DAC pwoer
    for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
        _FecRegister.FillDacControl(Fec, TFecRegister::cmdPowerUp, 0);
    }
    WriteFecRegister(PortFlags, _FecRegister.Bits);
    usleep(80);
    ReadFecRegister(PortFlags, 0);

    // set range to 'Wide'
    for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
        _FecRegister.FillDacControl(Fec, TFecRegister::cmdControl, 0x540);
    }
    WriteFecRegister(PortFlags, _FecRegister.Bits);
    usleep(80);
    ReadFecRegister(PortFlags, 0);

    return 1;
}


//// DP (80057) Methods ////

void TVmeMisc_ClearPulse_80057::ResetDpFifo(void)
{
    PutDoubleWord(regDpCommand, bitFifoReset | _CurrentDpTriggerMode);
}

void TVmeMisc_ClearPulse_80057::SetDpBusyOn(void)
{
    PutDoubleWord(regDpCommand, bitBusyOn | _CurrentDpTriggerMode);
}

void TVmeMisc_ClearPulse_80057::SetDpBusyOff(void)
{
    PutDoubleWord(regDpCommand, bitBusyOff | _CurrentDpTriggerMode);
}

void TVmeMisc_ClearPulse_80057::IssueDpTrigger(void)
{
    PutDoubleWord(regDpCommand, bitManualTrigger | _CurrentDpTriggerMode);
}

void TVmeMisc_ClearPulse_80057::SetDpTriggerMode(unsigned TriggerMode)
{
    _CurrentDpTriggerMode = TriggerMode;
    PutDoubleWord(regDpCommand, _CurrentDpTriggerMode);
}


//// PMT (80195) Methods ////

void TVmeMisc_ClearPulse_80057::IssuePmtTrigger(int PortFlags)
{
    WritePmtCommand(PortFlags, cmdTrigger);
}

void  TVmeMisc_ClearPulse_80057::SetPmtTriggerMode(int PortFlags, unsigned TriggerMode)
{
    _CurrentPmtTriggerMode = TriggerMode;
    WritePmtCommand(PortFlags, cmdSetTrigMode | _CurrentPmtTriggerMode);
}

void  TVmeMisc_ClearPulse_80057::SetPmtBusyExtension(int PortFlags, unsigned BusyExtension)
{
    WritePmtCommand(PortFlags, (cmdSetBusyExtension | BusyExtension) << 8);
}

void TVmeMisc_ClearPulse_80057::SetPmtHoldTime(int PortFlags, unsigned HoldTime)
{
    WritePmtCommand(PortFlags, (cmdSetHoldTime | HoldTime) << 8);
}

void TVmeMisc_ClearPulse_80057::SetPmtDynodeThreshold(int PortFlags, unsigned DacValue)
{
    unsigned Command = cmdSetDynodeThresholdData;
    unsigned Address = 0x4e;
    unsigned Arg1 = 0x0;
    unsigned Arg2 = DacValue;

    Command <<= 7; Command |= Address & 0x7f;
    Command <<= 8; Command |= Arg1 & 0xff;
    Command <<= 8; Command |= Arg2 & 0xff;
    WritePmtCommand(PortFlags, Command);
    WritePmtCommand(PortFlags, cmdWriteDynodeThresholdData);
    usleep(210);
}

void TVmeMisc_ClearPulse_80057::SetPmtGain(int PortFlags, unsigned ControlValue)
{
    // ControlValules: 0x140 -> Narrow, 0x540 -> Wide

    for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
        _FecRegister.FillDacControl(Fec, TFecRegister::cmdControl, ControlValue);
    }
    WriteFecRegister(PortFlags, _FecRegister.Bits);
    usleep(80);
    ReadFecRegister(PortFlags, 0);
}

void TVmeMisc_ClearPulse_80057::SetPmtAnodeTriggerChannelEnable(unsigned Channel)
{
    int Port = Channel / 64;
    int FecIndex = _FecRegister.FecIndexOf(Channel % 64);
    int DacChannel = _FecRegister.DacChannelOf(Channel % 64);
    _FecRegister.FillTriggerEnable(FecIndex, DacChannel, 1);
    for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
        _FecRegister.FillDacControl(Fec, TFecRegister::cmdNop, 0);
    }
    WriteFecRegister(0x0001 << Port, _FecRegister.Bits);
    usleep(80);
    ReadFecRegister(0x0001 << Port, 0);
}

void TVmeMisc_ClearPulse_80057::SetPmtAnodeThreshold(unsigned Channel, unsigned DacValue)
{
    int Port = Channel / 64;
    int FecIndex = _FecRegister.FecIndexOf(Channel % 64);
    int DacChannel = _FecRegister.DacChannelOf(Channel % 64);
    _FecRegister.FillDacThreshold(FecIndex, DacChannel, DacValue);
    WriteFecRegister(0x0001 << Port, _FecRegister.Bits);
    usleep(80);
    ReadFecRegister(0x0001 << Port, 0);    
}
