/* module-ClearPulse_80057.hh */
/* Created by Enomoto Sanshiro on 6 April 2015. */

#ifndef __module_ClearPulse_80057_hh__
#define __module_ClearPulse_80057_hh__

#include <string>
#include "RoomVmeAccess.hh"


class TVmeMisc_ClearPulse_80057: public TRoomVmeModule {
  public:    
    TVmeMisc_ClearPulse_80057(void);
    virtual ~TVmeMisc_ClearPulse_80057();
    virtual TRoomVmeModule* Clone(void);
    virtual bool Probe(void);
    virtual int NumberOfChannels(void);
    virtual int AddressBitLength(void);
    virtual int DataBitLength(void);
    virtual bool ProcessCommand(const std::string& Command, const std::vector<std::string>& ArgumentList);
    virtual int Initialize(int InitialState);
    virtual int Finalize(int FinalState);
    virtual int WriteRegister(int Address, int Data);
    virtual int ReadRegister(int Address, int& Data);
    virtual bool WaitData(unsigned TimeOut_sec);
    virtual bool HasData(int Address);
    virtual void ClearServiceRequest(void) {}
    virtual int Read(int Address, int &Data);
    virtual int NextDataBlockSize(int Address);
    virtual int BlockRead(int Address, void* Data, int MaxSize);
    virtual int MiscControlIdOf(const std::string& Command);
    virtual int MiscControl(int ControlId, int* ArgumentList, int NumberOfArguments);
  public:
    // 80057 - 80190 access methods //
    virtual int WritePmtCommand(int PortFlags, unsigned Command);
    virtual int WriteFecRegister(int PortFlags, const char* RegisterBits);
    virtual int ReadFecRegister(int PortFlags, unsigned int* Data);
    virtual int ResetFecRegister(int PortFlags);
    // DP (80057) methods //
    virtual void ResetDpFifo(void);
    virtual void SetDpBusyOn(void);
    virtual void SetDpBusyOff(void);
    virtual void IssueDpTrigger(void);
    virtual void SetDpTriggerMode(unsigned TriggerMode);
    // PMT (80190) methods //
    virtual void IssuePmtTrigger(int PortFlags);
    virtual void SetPmtTriggerMode(int PortFlags, unsigned TriggerMode);
    virtual void SetPmtBusyExtension(int PortFlags, unsigned BusyExtension);
    virtual void SetPmtHoldTime(int PortFlags, unsigned HoldTime);
    virtual void SetPmtDynodeThreshold(int PortFlags, unsigned DacValue);
    virtual void SetPmtGain(int PortFlags, unsigned ControlValue);
    virtual void SetPmtAnodeTriggerChannelEnable(unsigned Channel);
    virtual void SetPmtAnodeThreshold(unsigned Channel, unsigned DacValue);
  protected:
    enum TMiscControlId {
        ControlId_ResetDpFifo,
        ControlId_SetDpBusyOn,
        ControlId_SetDpBusyOff,
	ControlId_IssueDpTrigger,
        ControlId_EnableDpExtTrigIn,
        ControlId_EnableDpExtTrigOut,
	ControlId_IssuePmtTrigger,
	ControlId_SetPmtHoldTime,
	ControlId_SetPmtGainHigh,
	ControlId_SetPmtGainLow,
	ControlId_EnablePmtExtTrigOut,
	ControlId_EnablePmtExtTrigIn,
	ControlId_EnablePmtAnodeTrigger,
	ControlId_EnablePmtDynodeTrigger,
        ControlId_SetPmtDynodeThreshold,
        ControlId_SetPmtAnodeTriggerChannelEnable,
        ControlId_SetPmtAnodeThreshold,
	_NumberOfMiscControls
    };
    enum TVmeParameter {
	MapSize = 0x18000,
	TransferMode = VmeTransferMode_A24D32,
	VendorId = 0x43504350,
	KnownModelNumber = 80057,
	KnownFirmwareVersion = 4,
	_NumberOfVmeParameters
    };
    enum TRegister {
        regVendorId = 0x0000,
	regModuleId = 0x0004,
	regDpCommand = 0x0008,
	regTWidth = 0x0010,
	regVWidth = 0x0014,
	regPortMask = 0x0018,
	regBWidth = 0x001c,
	regStatus = 0x0020,
	regRealTime = 0x0024,
	regLiveTime = 0x0028,
	regDpPipeLine = 0x0030,
	regCommandA = 0x0044,
	regCommandB = 0x0048,
	regCommandC = 0x0050,
	regCommandD = 0x0060,
	regHeadSizeA = 0x0100,
	regHeadSizeB = 0x0104,
	regHeadSizeC = 0x0108,
	regHeadSizeD = 0x010c,
	regSdsBlockSizeA = 0x0110,
	regSdsBlockSizeB = 0x0114,
	regSdsBlockSizeC = 0x0118,
	regSdsBlockSizeD = 0x011c,
	regHeaderIdA = 0x0120,
	regHeaderIdB = 0x0124,
	regHeaderIdC = 0x0128,
	regHeaderIdD = 0x012c,
	regDataA = 0x11000,
	regDataB = 0x11400,
	regDataC = 0x11800,
	regDataD = 0x11c00,
	regPedestalA = 0x14000,
	regPedestalB = 0x14800,
	regPedestalC = 0x15000,
	regPedestalD = 0x15800,
	regGainA = 0x16000,
	regGainB = 0x16800,
	regGainC = 0x17000,
	regGainD = 0x17800,
	_NumberOfRegisters
    };
    enum TRegisterBit {
        // Command Register //
        bitFifoReset = 0x0001 << 0,
        bitManualTrigger = 0x0001 << 1,
	bitBusyOn = 0x0001 << 2,
	bitBusyOff = 0x0001 << 3,
	bitExtTrigIn = 0x0001 << 4,
	bitExtTrigOut = 0x0001 << 5,
	bitExtBusyOut = 0x0001 << 6,
	bitLedOn = 0x0001 << 12,
	bitLedOff = 0x0001 << 13,
	bitPowerOn = 0x0001 << 14,
	bitPowerOff = 0x0001 << 15,
        // Status Register //
	bitBusy = 0x01 << 8,
	bitCommandBusy = 0x01 << 9,
	_NumberOfRegisterBits
    };
    enum TPmtCommand {
        cmdReset = 0x1ff00000,
	cmdResetTime = 0x01010000,
	cmdTrigger = 0x01020000,
	cmdClear = 0x1040000,
	cmdSetTrigMode = 0x1060000,
	cmdSetDynodeThresholdData = 0x0000010b,
	cmdWriteDynodeThresholdData = 0x00010500,
	cmdSetHoldTime = 0x107000,
	cmdSetBusyExtension = 0x10800,
	cmdWriteRegister = 0x0109,
	_NumberOfPmtCommands
    };
    enum TPmtBits {
        // trigger mode //
        trigExtHoldOut = 0x4000,
        trigExtHoldin = 0x2000,
        trigExtTrigOut = 0x1000,
        trigExtTrigIn = 0x0800,
        trigAnodeTrig = 0x0400,
        trigDynodeTrig = 0x0200,
        trigSelfTrig = 0x0100,
        gainHigh = 0x140,
        gainLow = 0x540,
        _NumberOfPmtBits
    };
    enum TParameter {
        _NumberOfPorts = 4,
        _NumberOfChannels = 64,
        _NumberOfAllPortChannels = 256,
	_AddressBitLength = 8,
	_DataBitLength = 16,
	_NumberOfFecs = 4,
	_FecRegisterLength = 192,
	_DataBlockSize = 0x400,
	DataValue_Empty = 0xffffffff,
	DataValue_Head = 0xeb900000,
	DataHeadMask = 0xfff00000,
	_NumberOfParameters
    };
  protected:
    struct TFecRegister {
        char Bits[_FecRegisterLength];
        TFecRegister(void) {
            for (unsigned Fec = 0; Fec < _NumberOfFecs; Fec++) {
                FillFpga(Fec, 1);
                FillDacControl(Fec, TFecRegister::cmdNop, 0);
            }
        }
        void FillFpga(int FecIndex, int Enabled) {
            for (int Bit = 0; Bit < 24; Bit++) {
                Bits[FecIndex * 48 + Bit] = 0;
	    };
	    Bits[FecIndex * 48 + 7] = Enabled;
	    Bits[FecIndex * 48 + 2] = ((FecIndex == 0) || (FecIndex == 3)) ? 1 : 0;
	}
        void FillTriggerEnable(int FecIndex, int Channel, int Enabled) {
            Bits[FecIndex * 48 + (23-Channel)] = Enabled;
	}
        void FillDacThreshold(int FecIndex, int Channel, int Threshold) {
            int BitValue;
            for (int Bit = 0; Bit < 24; Bit++) {
	        if (Bit < 2) BitValue = 0;
		else if (Bit < 14) BitValue = (Threshold >> (Bit-2) & 0x01);
		else if (Bit < 16) BitValue = 1;
		else if (Bit < 20) BitValue = (Channel >> (Bit-16) & 0x01);
		else BitValue = 0;
		Bits[FecIndex * 48 + 24 + (23-Bit)] = BitValue;
	    };
	}
        void FillDacControl(int FecIndex, int Command, int Value) {
            int BitValue;
            for (int Bit = 0; Bit < 24; Bit++) {
	        if (Bit < 2) BitValue = 0;
		else if (Bit < 14) BitValue = (Value >> (Bit-2) & 0x01);
		else if (Bit < 16) BitValue = 0;
		else if (Bit < 20) BitValue = (Command >> (Bit-16) & 0x01);
		else BitValue = 0;
		Bits[FecIndex * 48 + 24 + (23-Bit)] = BitValue;
	    };
	}
        int FecIndexOf(int PmtChannel) {
            static const int Table[] = {
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1,
                2,2,3,3,0,0,1,1
            };
            return Table[PmtChannel];
        }
        int DacChannelOf(int PmtChannel) {
            static const int Table[] = {
                15,14,15,14,15,14,15,14,
                13,12,13,12,13,12,13,12,
                11,10,11,10,11,10,11,10,
                 9, 8, 9, 8, 9, 8, 9, 8,
                 7, 6, 7, 6, 7, 6, 7, 6,
                 5, 4, 5, 4, 5, 4, 5, 4,
                 3, 2, 3, 2, 3, 2, 3, 2,
                 1, 0, 1, 0, 1, 0, 1, 0
            };
            return Table[PmtChannel];
        }
        enum TCommand {
            cmdReset = 0x0f,
            cmdPowerDown = 0x08,
            cmdPowerUp = 0x09,
            cmdControl = 0x0c,
            cmdNop = 0x00,
	    _NumberOfCommands
	};
    };
  protected:
    int _ModelNumber, _FirmwareVersion;
    int _SingleReadBuffer[_NumberOfAllPortChannels];
    bool _IsSnigleReadDataReady;
    unsigned _EnabledPortFlags;
    unsigned _CurrentDpTriggerMode, _CurrentPmtTriggerMode;
    TFecRegister _FecRegister;  // the last-written values are held here.
};


#endif
