/* module-Rinei_RPV130.hh */
/* Created by Enomoto Sanshiro on 26 May 1997. */
/* Last updated by Enomoto Sanshiro on 29 December 2002. */


#ifndef __module_Rinei_RPV130_hh__
#define __module_Rinei_RPV130_hh__


#include "RoomModule.hh"
#include "RoomVmeAccess.hh"


/* Interrupt & I/O Register Model RPV-130 (Rinei) */
class TVmeIORegister_Rinei_RPV130: public TRoomVmeModule {
  public:
    TVmeIORegister_Rinei_RPV130(void);
    virtual ~TVmeIORegister_Rinei_RPV130();
    virtual TRoomVmeModule* Clone(void);
  public:
    virtual int Initialize(int InitialState = 0);
    virtual int Finalize(int FinalState = 0);
    virtual int Enable(int Address = -1);
    virtual int Disable(int Address = -1);
    virtual int IsEnabled(int Address = -1);
    virtual int IsBusy(int Address = -1);
    virtual int Clear(int Address = -1);
    virtual int WriteRegister(int Address, int Data);
    virtual int ReadRegister(int Address, int& Data);
    virtual int EnableInterrupt(int SignalId = 0);
    virtual int DisableInterrupt(void);
    virtual int ClearInterrupt(void);
    virtual bool HasData(int Address = -1);
    virtual bool WaitData(unsigned TimeOut_sec);
  public:
    virtual void EnableServiceRequest(void);
    virtual void DisableServiceRequest(void);
    virtual void ClearServiceRequest(void);
    virtual bool IsRequestingService(void);
    virtual int RequestingServiceNumber(void);
    virtual void EnableSignalOnServiceRequest(int SignalId);
    virtual void DisableSignalOnServiceRequest(void);
  public:
    virtual int MiscControlIdOf(const std::string& CommandName);
    virtual int MiscControl(int ControlId, int* ArgumentList = 0, int NumberOfArguments = 0);
    int InputLatch1(int& Data);
    int InputLatch2(int& Data);
    int InputFlipFlop(int& Data);
    int InputThrough(int& Data);
    int OutputLevel(int Data);
    int OutputPulse(int Data);
  public:
    enum TControlId {
	ControlId_InputLatch1,
	ControlId_InputLatch2,
	ControlId_InputFlipFlop,
	ControlId_InputThrough,
	ControlId_OutputLevel,
	ControlId_OutputPulse,
	_NumberOfControls
    };
  private:
    enum TRPV130Parameters {
	RPV130_MemorySize = 0x1000,
	RPV130_TransferMode = VmeTransferMode_A16D16
    };
    enum TRPV130Symbols {
	regInputLatch1 = 0x0000,
	regInputLatch2 = 0x0002,
	regInputFlipFlop = 0x0004,
	regInputThrough = 0x0006,
	regOutputPulse = 0x0008,
	regOutputLevel = 0x000a,
	regCSR1 = 0x000c,
	regCSR2 = 0x000e,
	    bitBusy3 = 0x80,
	    bitEnable3 = 0x40,
	    bitBusy12 = 0x20,
	    bitEnable12 = 0x10,
	    bitMask12 = 0x08,
	    bitClear12 = 0x02,
	    bitClear3 = 0x01
    };
  private:
    int _EnabledChannelBits;
};


#endif
