/* module-SiTCP.cc */
/* Created by Enomoto Sanshiro on 31 July 1999. */
/* Last updated by Enomoto Sanshiro on 4 November 2001. */

#include "RoomModule.hh"
#include "RoomNetworkAccess.hh"
#include "RoomDeviceFactory.hh"
#include "module-SiTCP.hh"
#include <arpa/inet.h>

using namespace std;


static TRoomNetworkModuleCreator Creator("Generic_SiTCP", new TNetworkMisc_SiTCP());


TNetworkMisc_SiTCP::TNetworkMisc_SiTCP(void)
: TRoomNetworkModule("NetworkMisc", "Generic_SiTCP"), _RegisterFileConnection(false)
{
    _TcpSocket = -1;
    _UdpSocket = -1;
    _SequenceCount = 0;

    _DataBufferSize = 2048;
    _DataBuffer = 0;
}

TNetworkMisc_SiTCP::~TNetworkMisc_SiTCP()
{
    delete[] _DataBuffer;
}

TRoomNetworkModule* TNetworkMisc_SiTCP::Clone(void)
{
    return new TNetworkMisc_SiTCP();
}

int TNetworkMisc_SiTCP::Connect(const std::string& RemoteHostName, int PortNumber)
{
    static const int DefaultTcpPortNumber = 24;
    static const int DefaultUdpPortNumber = 4660;
    
    _TcpSocket = TRoomNetworkModule::Connect(RemoteHostName, DefaultTcpPortNumber);
    _UdpSocket = _RegisterFileConnection.Connect(RemoteHostName, DefaultUdpPortNumber);

    if (! _DataBuffer) {
        _DataBuffer = new unsigned char[_DataBufferSize];
    }

#if 0
    cerr << "Conected to SiTCP" << endl;
    cerr << "  TCP File ID: " << _TcpSocket << endl;
    cerr << "  UDP File ID: " << _UdpSocket << endl;
#endif

    return _TcpSocket;
}

int TNetworkMisc_SiTCP::Transact(int Command, int Address, int& Data)
{
    int NumberOfBytes = 4;
    int PacketSize = 3 * sizeof(U32bit);
    
    _DataBuffer[0] = 0xff;
    _DataBuffer[1] = Command;
    _DataBuffer[2] = _SequenceCount++;
    _DataBuffer[3] = NumberOfBytes;
    ((U32bit*)_DataBuffer)[1] = htonl((unsigned) Address);
    ((U32bit*)_DataBuffer)[2] = htonl((unsigned) Data);
    
    int SentSize = _RegisterFileConnection.SendBlock(_DataBuffer, PacketSize);
    if (SentSize < PacketSize) {
	throw THardwareException(
	    "TNetworkMisc_SiTCP::Transact()", "unable to send request block"
        );
    }

    int ReceivedSize = _RegisterFileConnection.ReceiveBlock(_DataBuffer, PacketSize);
    if (ReceivedSize < PacketSize) {
	throw THardwareException(
	    "TNetworkMisc_SiTCP::Transact()", "received packet too short"
        );
    }
    if ((_DataBuffer[1] & 0x0f) != 0x08) {
	throw THardwareException(
	    "TNetworkMisc_SiTCP::Transact()", "SiTCP bus error"
        );
    }
        
    if (Command == Command_Read) {
        Data = ntohl(((U32bit*)_DataBuffer)[2]);
    }

#if 0
    for (int i = 0; i < ReceivedSize; i++) {
        cerr << hex << (int) _DataBuffer[i] << "  ";
    }
    cerr << dec << endl;
#endif
    
    return ReceivedSize;
}

int TNetworkMisc_SiTCP::WriteRegister(int Address, int Data)
{
    return (Transact(Command_Write, Address, Data) > 0) ? 1 : 0;
}

int TNetworkMisc_SiTCP::ReadRegister(int Address, int& Data)
{
    return (Transact(Command_Read, Address, Data) > 0) ? 1 : 0;
}

int TNetworkMisc_SiTCP::BlockRead(int Address, void *Data, int MaxSize)
{
    return TRoomNetworkModule::ReceiveAvailable(Data, MaxSize);
}

int TNetworkMisc_SiTCP::BlockWrite(int Address, const void *Data, int Size)
{
    return TRoomNetworkModule::SendBlock(Data, Size);
}

int TNetworkMisc_SiTCP::NextDataBlockSize(int Address)
{
    return 1024*1024;
}
