/* module-SimulatedModules.hh */
/* Created by Enomoto Sanshiro on 23 April 1999. */
/* Last updated by Enomoto Sanshiro on 8 July 2001. */


#ifndef __module_SimulatedModules_hh__
#define __module_SimulatedModules_hh__


#include "RoomDefs.hh"
#include "RoomModule.hh"
#include "RoomSoftwareDevice.hh"


class TPoissonGenerator: public TRoomSoftwareModule {
  public:
    TPoissonGenerator(int NumberOfChannels = 16, int FullScale = 4096);
    virtual ~TPoissonGenerator();
    virtual TRoomSoftwareModule* Clone(void);
    virtual int Clear(int Address = -1);
    virtual int Read(int Address, int &Data);
    virtual int NumberOfChannels(void);
  private:
    int _NumberOfChannels;
    int _FullScale;
    double _Mean;
};


class TGaussianGenerator: public TRoomSoftwareModule {
  public:
    TGaussianGenerator(int NumberOfChannels = 16, int FullScale = 4096);
    virtual ~TGaussianGenerator();
    virtual TRoomSoftwareModule* Clone(void);
    virtual int Clear(int Address = -1);
    virtual int Read(int Address, int &Data);
    virtual int NumberOfChannels(void);
  private:
    int _NumberOfChannels;
    int _FullScale;
    double _Mean, _Deviation;
};


class TExponentialGenerator: public TRoomSoftwareModule {
  public:
    TExponentialGenerator(int NumberOfChannels = 16, int FullScale = 4096);
    virtual ~TExponentialGenerator();
    virtual TRoomSoftwareModule* Clone(void);
    virtual int Clear(int Address = -1);
    virtual int Read(int Address, int &Data);
    virtual int NumberOfChannels(void);
  private:
    int _NumberOfChannels;
    int _FullScale;
    double _Lambda;
};


class TWaveGenerator: public TRoomSoftwareModule {
  public:
    TWaveGenerator(int NumberOfChannels = 16, int NumberOfSamples = 128, int FullScale = 4096);
    virtual ~TWaveGenerator();
    virtual TRoomSoftwareModule* Clone(void);
    virtual int Clear(int Address = -1);
    virtual int Read(int Address, int &Data);
    virtual int NumberOfChannels(void);
    virtual int AddressBitLength(void);
    virtual int ElementDepth(void);
    virtual int NextNumberOfDataElements(int Address = -1);
    virtual int MiscControlIdOf(const std::string& CommandName);
    virtual int MiscControl(int ControlId, int* ArgumentList = 0, int NumberOfArguments = 0);
  private:
    int _NumberOfChannels;
    int _NumberOfSamples;
    int *_ReadoutCount;
    int _FullScale;
    double _NoiseLevel;
  public:
    enum TContolId {
        ControlId_SetSamplingDepth,
        ControlId_SetFullScale,
        ControlId_SetNoiseLevel,
	_NumberOfControls
    };
};


class TPmtSimulator: public TRoomSoftwareModule {
  public:
    TPmtSimulator(int FullScale = 4096);
    virtual ~TPmtSimulator();
    virtual TRoomSoftwareModule* Clone(void);
    virtual int Initialize(int InitialState);
    virtual int Clear(int Address = -1);
    virtual bool HasData(int Address);
    virtual bool WaitData(unsigned TimeOut_sec);
    virtual int Read(int Address, int &Data);
    virtual int NumberOfChannels(void);
    virtual int AddressBitLength(void);
    virtual int DataBitLength(void);
  public:
    virtual int MiscControlIdOf(const std::string& CommandName);
    virtual int MiscControl(int ControlId, int* ArgumentList = 0, int NumberOfArguments = 0);
    virtual void ClearServiceRequest(void);
  protected:
    void SetEventRate(long Rate);
    double GetCurrentTime(void);
  private:
    int _FullScale;
    long _EpochTime;
    double _EventRate, _NextEventTime;
  public:
    enum TContolId {
	ControlId_SetEventRate,
	_NumberOfControls
    };
};


#endif

