/* KinokoDataAnalyzer.hh */
/* Created by Enomoto Sanshiro on 23 October 1999. */
/* Last updated by Enomoto Sanshiro on 2 September 2001. */


#ifndef __KinokoDataAnalyzer_hh__
#define __KinokoDataAnalyzer_hh__


#include "KinokoDataSource.hh"
#include "KinokoDataSection.hh"
#include "KinokoBlockDataSection.hh"
#include "KinokoIndexedDataSection.hh"
#include "KinokoTaggedDataSection.hh"
#include "KinokoNestedDataSection.hh"


class TKinokoDataAnalyzer {
  public:
    TKinokoDataAnalyzer(void);
    virtual ~TKinokoDataAnalyzer();
    virtual void Destroy(void) {}
    virtual bool IsSharable(void);
    virtual void ReadDataSource(TKinokoDataSource* DataSource);
    virtual int ProcessDataPacket(void* DataPacket, TKinokoDataSource* DataSource = 0, TKinokoDataSection* DataSection = 0);
    virtual int ProcessTrailerPacket(void* Packet, TKinokoDataSource* DataSource = 0);
    virtual int ProcessCommandPacket(void* Packet, TKinokoDataSource* DataSource = 0);
    virtual void Reset(void);
    virtual void Dump(std::ostream& os, const std::string& Indent = "");
  protected:
    virtual int ProcessIndexedData(void* DataPacket, TKinokoIndexedDataSection* DataSection);
    virtual int ProcessTaggedData(void* DataPacket, TKinokoTaggedDataSection* DataSection);
    virtual int ProcessBlockData(void* DataPacket, TKinokoBlockDataSection* DataSection);
    virtual int ProcessNestedData(void* DataPacket, TKinokoNestedDataSection* DataSection);
  private: 
    TKinokoDataSource* _DataSource;
};


class TKinokoDataAnalyzerList: public TKinokoDataAnalyzer {
  public:
    TKinokoDataAnalyzerList(void);
    virtual ~TKinokoDataAnalyzerList();
    virtual void Destroy(void);
    virtual void ReadDataSource(TKinokoDataSource* DataSource);
    virtual int ProcessDataPacket(void* DataPacket, TKinokoDataSource* DataSource = 0, TKinokoDataSection* DataSection = 0);
    virtual int ProcessTrailerPacket(void* Packet, TKinokoDataSource* DataSource = 0);
    virtual int ProcessCommandPacket(void* Packet, TKinokoDataSource* DataSource = 0);
    virtual void Reset(void);
    virtual void Dump(std::ostream& os, const std::string& Indent = "");
  public:
    virtual void AddAnalyzer(TKinokoDataAnalyzer* Analyzer);
    virtual void DestroyAnalyzer(void);
  protected:
    TKinokoDataAnalyzer** _AnalyzerList;
    int _NumberOfAnalyzers;
    int _AnalyzerListCapacity;
};


#endif
