/* KinokoDataProcessor.hh */
/* Created by Enomoto Sanshiro on 20 April 2001. */
/* Last updated by Enomoto Sanshiro on 21 September 2004. */


#ifndef __KinokoDataProcessor_hh__
#define __KinokoDataProcessor_hh__


#include <iostream>
#include <string>
#include "KinokoPlatform.hh"
#include "KinokoLogger.hh"
#include "KinokoStream.hh"
#include "KinokoDataStreamFormatter.hh"
#include "KinokoDataDescriptor.hh"
#include "KinokoDataSource.hh"
#include "KinokoDataStreamMonitor.hh"


class TKinokoDataSender {
  public:
    TKinokoDataSender(void);
    virtual ~TKinokoDataSender();
    virtual void ConstructOutlet(TKinokoOutputStream* OutputStream, TKinokoDataSource* DataSource);
  public:
    virtual void SendDataDescriptorPacket(void);
    virtual void SendRunBeginPacket(int Timestamp = 0);
    virtual void SendRunEndPacket(int Timestamp = 0);
    virtual void SendClockTickPacket(int Timestamp);
    virtual void SendCommandPacket(int Command, int Timestamp = 0);
    virtual void SendEventTrailerPacket(int Timestamp = 0);
    virtual void SendPacket(void* Packet, long PacketSize);
    virtual void* AllocatePacket(long PacketSize);
    virtual void FlushPacket(void* Packet, long PacketSize);
  protected:
    TKinokoOutputStream* _OutputStream;
    TKinokoDataStreamFormatter* _OutputStreamFormatter;
    TKinokoDataDescriptor* _OutputDataDescriptor;
    TKinokoDataSource* _OutputDataSource;
    int _LastClockTick;
};


class TKinokoDataReceiver {
  public:
    TKinokoDataReceiver(void);
    virtual ~TKinokoDataReceiver();
    virtual void ConstructInlet(TKinokoInputStream* InputStream);
    virtual void SetStreamCommandProcessor(TKinokoStreamCommandProcessor* Processor);
    virtual TKinokoDataDescriptor* InputDataDescriptor(void);
    virtual TKinokoDataStreamMonitor* DataStreamMonitor(void);
    virtual void SetChainedOutput(TKinokoOutputStream* OutputStream);
  public:
    virtual int ProcessInputStream(void);
    virtual void ProcessPacket(void* Packet, long PacketSize);
    virtual bool ProcessDataDescriptorPacket(void* Packet, long PacketSize);
    virtual bool ProcessCommandPacket(void* Packet, long PacketSize);
    virtual bool ProcessDataPacket(void* Packet, long PacketSize);
    virtual bool ProcessTrailerPacket(void* Packet, long PacketSize);
  protected:
    virtual void OnRunBegin(void) {}
    virtual void OnRunEnd(void) {}
    virtual void OnReceivePacket(void* Packet, long PacketSize) {}
    virtual void OnReceiveDataDescriptorPacket(void* Packet, long PacketSize) {}
    virtual void OnReceiveCommandPacket(void* Packet, long PacketSize) {}
    virtual void OnReceiveDataPacket(void* Packet, long PacketSize) {}
    virtual void OnReceiveTrailerPacket(void* Packet, long PacketSize) {}
    virtual void OnReceiveEventTrailerPacket(void* Packet, long PacketSize) {}
  protected:
    TKinokoInputStream* _InputStream;
    TKinokoDataStreamScanner* _InputStreamScanner;
    TKinokoDataDescriptor* _InputDataDescriptor;
    TKinokoOutputStream* _ChainedOutputStream;
    TKinokoDataStreamMonitor* _DataStreamMonitor;
  private:
    TKinokoStreamCommandProcessor* _StreamCommandProcessor;
    TKinokoStreamCommandProcessor* _DefaultStreamCommandProcessor;
};


class TKinokoDataProducer: public TKinokoDataSender {
  public:
    TKinokoDataProducer(void);
    virtual ~TKinokoDataProducer();
    virtual void AttachPlatform(std::istream& InputIoStream, std::ostream& OutputIoStream, TKinokoEventEmitter* EventEmitter, TKinokoRegistry* Registry, TKinokoLogger* Logger);
    virtual void Construct(const std::string& Name, TKinokoOutputStream* OutputStream, int DataSourceId);
    virtual void Destruct(void);
    virtual int ProcessData(void);
  protected:
    virtual void BuildDataSource(TKinokoDataSource* DataSource) = 0;
    virtual void OnConstruct(void) {}
    virtual void OnDestruct(void) {}
  protected:
    std::istream* _InputIoStream;
    std::ostream* _OutputIoStream;
    TKinokoEventEmitter* _EventEmitter;
    TKinokoRegistry* _Registry;
    TKinokoLogger* _Logger;
};


class TKinokoDataProcessor: public TKinokoDataReceiver, public TKinokoDataSender {
  public:
    TKinokoDataProcessor(void);
    virtual ~TKinokoDataProcessor();
    virtual void AttachPlatform(std::istream& InputIoStream, std::ostream& OutputIoStream, TKinokoEventEmitter* EventEmitter, TKinokoRegistry* Registry, TKinokoLogger* Logger);
    virtual void Construct(const std::string& Name, TKinokoInputStream* InputStream, TKinokoOutputStream* OutputStream, int DataSourceId);
    virtual void Destruct(void);
    virtual int ProcessData(void);
  protected:
    virtual void BuildDataSource(TKinokoDataSource* DataSource) = 0;
    virtual void OnConstruct(void) {}
    virtual void OnDestruct(void) {}
  protected:
    std::istream* _InputIoStream;
    std::ostream* _OutputIoStream;
    TKinokoEventEmitter* _EventEmitter;
    TKinokoRegistry* _Registry;
    TKinokoLogger* _Logger;
};


class TKinokoDataConsumer: public TKinokoDataReceiver {
  public:
    TKinokoDataConsumer(void);
    virtual ~TKinokoDataConsumer();
    virtual void AttachPlatform(std::istream& InputIoStream, std::ostream& OutputIoStream, TKinokoEventEmitter* EventEmitter, TKinokoRegistry* Registry, TKinokoLogger* Logger);
    virtual void Construct(const std::string& Name, TKinokoInputStream* InputStream);
    virtual void Destruct(void);
    virtual int ProcessData(void);
  protected:
    virtual void OnConstruct(void) {}
    virtual void OnDestruct(void) {}
  protected:
    std::istream* _InputIoStream;
    std::ostream* _OutputIoStream;
    TKinokoEventEmitter* _EventEmitter;
    TKinokoRegistry* _Registry;
    TKinokoLogger* _Logger;
};


#endif
