/* KinokoWaveWriter.cc */
/* Created by Enomoto Sanshiro on 2 April 2001. */
/* Last updated by Enomoto Sanshiro on 27 June 2002. */


#include <string>
#include <algorithm>
#include "KameTabular.hh"
#include "KinokoSectionDataAnalyzer.hh"
#include "KinokoWaveWriter.hh"

using namespace std;
using namespace kame;


TKinokoWaveWriter::TKinokoWaveWriter(TKameTabular* Tabular, const string& SectionPath, const TKinokoDataElementSpecifier& ElementSpecifier)
{
    _Tabular = Tabular;
    _SectionPath = SectionPath;
    _ElementSpecifier = ElementSpecifier;

    AddSection(SectionPath);

    _TargetName = "untitled";
}

TKinokoWaveWriter::~TKinokoWaveWriter()
{
}

void TKinokoWaveWriter::Destroy(void)
{
}

void TKinokoWaveWriter::Reset(void)
{
    _Tabular->Clear();
}

int TKinokoWaveWriter::ProcessData(TKinokoSectionData* SectionData)
{
    int DataIndex = 0;
    int Address; float Data;

    while (SectionData->GetNextOf(_ElementSpecifier, Address, Data)) {
#if 1
	//... for backward compatibility ...//
	Data = _ElementSpecifier.ConvertData(Data);
#endif
        try {
            if ((*_Tabular)[DataIndex][0].IsVoid()) {
                (*_Tabular)[DataIndex][0] = Data;
            }
            else {
                (*_Tabular)[DataIndex][0] = (*_Tabular)[DataIndex][0].As<float>() + Data;
            }
            if ((*_Tabular)[DataIndex][1].IsVoid()) {
                (*_Tabular)[DataIndex][1] = 1;
            }
            else {
                (*_Tabular)[DataIndex][1] = (*_Tabular)[DataIndex][1].As<float>() + 1;
            }
        }
        catch (TKameException &e) {
            cerr<< "ERROR: "<< e << endl;
        }
        
	DataIndex++;
    }

    return (DataIndex > 0) ? 1 : 0;
}

void TKinokoWaveWriter::SetTargetName(const std::string& Name)
{
    _TargetName = Name;
}

void TKinokoWaveWriter::Dump(ostream& os, const string& Indent)
{
    os << Indent << "FillWave" << "\t" << _TargetName;
    os << ", \"" << _SectionPath << "\", " << _ElementSpecifier.AsString();
    os << endl;
}
