/* KinokoBuilder.hh */
/* Created by Enomoto Sanshiro on 25 August 2009. */
/* Updated by Enomoto Sanshiro on 19 May 2010. */
/* Last updated by Enomoto Sanshiro on 7 July 2010. */


#ifndef __KinokoBuilder_hh__
#define __KinokoBuilder_hh__


#include <string>
#include <iostream>
#include <vector>
#include "KinokoMemoryPool.hh"
#include "KinokoDataProcessor.hh"
#include "KinokoEventSorter.hh"
#include "KinokoEventEnvelope.hh"
#include "KinokoSorterController.hh"
#include "KinokoBuilderMonitor.hh"


class TKinokoBuilder: public TKinokoDataProcessor, protected TKinokoEventPieceDefs {
  public:
    TKinokoBuilder(void);
    virtual ~TKinokoBuilder();
    virtual void SetOutputDataSourceName(const std::string& DataSourceName);
    virtual void SetBufferSize(size_t BufferSize);
    virtual void SetBuilderLogger(TKinokoBuilderLogger* Logger);
    virtual void SetBuilderMonitor(TKinokoBuilderMonitor* Monitor);
    virtual void SetStartDataSize(size_t StartDataSize);
    virtual void SetStartPacketCycles(int NumberOfPacketCycles);
    virtual void SetStartChannelList(const std::vector<long>& ChannelList);
    virtual void SetInputPreservation(bool IsInputPreserved);
    virtual void SetActivityWindowLength(int WindowLength);
    virtual void SetReportInterval(long Interval_sec);
  public:
    // DataProcessor Interface //
    virtual void Construct(const std::string& Name, TKinokoInputStream* InputStream, TKinokoOutputStream* OutputStream, int DataSourceId);
  protected:
    // DataProcessor Interface //
    virtual void OnConstruct(void);
    virtual void OnDestruct(void);
    virtual void BuildDataSource(TKinokoDataSource* DataSource);
    virtual void OnRunBegin(void);
    virtual void OnRunEnd(void);
    virtual void OnReceiveCommandPacket(void* Packet, long PacketSize);
    virtual void OnReceivePacket(void* Packet, long PacketSize);
  protected:
    // Builder Interface //
    virtual TKinokoEventSorter& BuilderConstruct(void) = 0;
    virtual void BuilderBegin(void) = 0;
    virtual void BuilderEnd(void) = 0;
    virtual int BuilderProcess(void* Packet, long PacketSize) = 0;
  protected:
    size_t _BufferSize;
    TKinokoMemoryPool* _MemoryPool;
    TKinokoBuilderLogger* _Logger;
    TKinokoBuilderMonitor* _Monitor;
    TKinokoSorterController* _SorterController;
  protected:
    size_t _StartDataSize;
    int _StartPacketCycles;
    std::vector<long> _StartChannelList;
    bool _IsInputPreserved;
  protected:
    std::string _OutputDataSourceName;
    TKinokoEventEnvelopeWriter* _EnvelopeWriter;
  private:
    TKinokoBuilderLogger* _MyLogger;
    TKinokoBuilderMonitor* _MyMonitor;
    long _ReportInterval, _LastReportTime;
};


#endif
