/* KinokoBuilderScannerTest.cc */
/* Edited by Enomoto Sanshiro on 2 September 2010. */
/* Last updated by Enomoto Sanshiro on 2 September 2010. */


#include <iostream>
#include "MushArgumentList.hh"
#include "KinokoBuilderInlet.hh"
#include "KinokoEventChopper.hh"
#include "KinokoBuilderConfiguration.hh"
#include "KiscFancyConfigFile.hh"
#include "KinokoBuilderScannerTester.hh"

using namespace std;


TKinokoBuilderScannerDumper::TKinokoBuilderScannerDumper(std::ostream& Output)
: _Output(Output) 
{
    _Output << "# Title: Sorter Dump" << endl;
    _Output << "# Fields: PieceType ChannelKey EventKey PieceFlag Size" << endl;
    _Output << endl;
}

TKinokoBuilderScannerDumper::~TKinokoBuilderScannerDumper() 
{
}

void TKinokoBuilderScannerDumper::DumpInputRunBegin(void)
{
    _Output << "# Command: RunBegin" << endl;
}

void TKinokoBuilderScannerDumper::DumpInputRunEnd(void)
{
    _Output << "# Command: RunEnd" << endl;
}

void TKinokoBuilderScannerDumper::DumpInputPacket(int SectionId, void* Data, int Size)
{
    _Output << "# Input: SectionId=" << SectionId << ", Size=" << Size << endl;
}

void TKinokoBuilderScannerDumper::DumpOutputPiece(const TKinokoEventPiece& EventPiece)
{
    switch (EventPiece.PieceType) {
      case PieceType_Normal:
	_Output << "N  ";
	break;
      case PieceType_Padding:
	_Output << "P  ";
	break;
      case PieceType_Unaligned:
	_Output << "U  ";
	break;
      case PieceType_Corrupted:
	_Output << "C  ";
	break;
      default:
	_Output << "?  ";
    }

    _Output << EventPiece.ChannelKey << "  ";
    _Output << EventPiece.EventKey << "  ";
    _Output << EventPiece.PieceFlag << "  ";
    _Output << EventPiece.Size() << endl;
}



class TKinokoBuilderScannerDumperAdapter: public TKinokoEventPieceProcessor {
  public:
    TKinokoBuilderScannerDumperAdapter(TKinokoBuilderScannerDumper* Dumper);
    virtual ~TKinokoBuilderScannerDumperAdapter();
    virtual int ProcessPiece(const TKinokoEventPiece& EventPiece);
  protected:
    TKinokoBuilderScannerDumper* _Dumper;
};

TKinokoBuilderScannerDumperAdapter::TKinokoBuilderScannerDumperAdapter(TKinokoBuilderScannerDumper* Dumper)
{
    _Dumper = Dumper;
}

TKinokoBuilderScannerDumperAdapter::~TKinokoBuilderScannerDumperAdapter()
{
}

int TKinokoBuilderScannerDumperAdapter::ProcessPiece(const TKinokoEventPiece& EventPiece) 
{
    _Dumper->DumpOutputPiece(EventPiece);
    return 1;
}



TKinokoBuilderScannerTester::TKinokoBuilderScannerTester(TKinokoEventPieceScanner* Scanner, TKinokoBuilderScannerDumper* Dumper)
{
    _Scanner = Scanner;
    _Dumper = Dumper;
    _OutputProcessor = new TKinokoBuilderScannerDumperAdapter(_Dumper);

    _Chopper = 0;
    _BuilderInlet = 0;
}

TKinokoBuilderScannerTester::~TKinokoBuilderScannerTester()
{
    delete _Chopper;
    delete _BuilderInlet;

    delete _OutputProcessor;
}

void TKinokoBuilderScannerTester::Configure(const std::string& ConfigFileName, const std::string& BuilderName)
{
    TKiscFancyConfigFile ConfigFile(ConfigFileName);
    ConfigFile.SetEntryKeyword("builder");
    ConfigFile.SetTargetSection(BuilderName);

    TKinokoKiscBuilderConfiguration BuilderConfig(ConfigFile);
    BuilderConfig.ConfigureDataSectionInlet(this);
}

void TKinokoBuilderScannerTester::OnConstruct(void)
{
    _Chopper = new TKinokoEventChopper(_OutputProcessor, _Scanner);
    _BuilderInlet = new TKinokoBuilderInlet(_Chopper);

    _BuilderInlet->SetTargetDataSource(_InputDataSourceName);
    for (unsigned i = 0; i < _InputDataSectionNameList.size(); i++) {
	_BuilderInlet->AddTargetDataSection(
	    _InputDataSectionNameList[i].first,
	    _InputDataSectionNameList[i].second
	);
    }
}

void TKinokoBuilderScannerTester::OnRunBegin(void) 
{
    _Dumper->DumpInputRunBegin();
}

void TKinokoBuilderScannerTester::OnRunEnd(void) 
{
    _Dumper->DumpInputRunEnd();
}

void TKinokoBuilderScannerTester::OnReceivePacket(void* Packet, long PacketSize)
{
    if (TKinokoDataStreamScanner::IsDataPacket(Packet)) {
	int SectionId = TKinokoDataSectionScanner::SectionIdOf(Packet);
	int DataSize = TKinokoDataSectionScanner::DataSizeOf(Packet);
	void* DataArea = TKinokoDataSectionScanner::DataAreaOf(Packet);
	_Dumper->DumpInputPacket(SectionId, DataArea, DataSize);
    }

    _BuilderInlet->ProcessPacket(Packet, PacketSize);
}




#if 0
int main(int argc, char** argv)
{
    TMushArgumentList ArgumentList(argc, argv);
    string ConfigFileName = ArgumentList["--config"];
    string BuilderName = ArgumentList["--name"];
    if ((ArgumentList.NumberOfParameters() < 1) ||
	ConfigFileName.empty() || BuilderName.empty()
    ){
	cerr << "Usage: " << ArgumentList.ProgramName();
	cerr << " --config=CONFIG_FILE_NAME --name=BUILDER_NAME";
	cerr << " INPUT_FILE" << endl;
	return -1;
    }       
    
    TMoguraEventPieceScanner Scanner;
    TKinokoBuilderScannerDumper Dumper(cout);
    TKinokoBuilderScannerTester Tester(&Scanner, &Dumper);
    TKinokoStandaloneDataConsumer StandaloneConsumer(&Tester, "ScannerTester");

    try {
	Tester.Configure(ConfigFileName, BuilderName);
	StandaloneConsumer.Start(ArgumentList);
    }
    catch (TKinokoException &e) {
	cerr << "ERROR: " << argv[0] << ": " << e << endl;
    }
    
    return 0;
}
#endif
