/* KinokoEventProcessor.cc */
/* Created by Enomoto Sanshiro on 4 September 2009. */
/* Last updated by Enomoto Sanshiro on 4 September 2009. */


#include <string>
#include <iostream>
#include "KinokoDataProcessor.hh"
#include "KinokoEventEnvelope.hh"
#include "KinokoEventProcessor.hh"

using namespace std;


TKinokoEventProcessor::TKinokoEventProcessor(const std::string& InputDataSourceName, TKinokoEventSegmentProcessor* SegmentProcessor)
{
    _SegmentProcessor = SegmentProcessor;
    _EnvelopeReader = new TKinokoEventEnvelopeReader(_SegmentProcessor);

    if (! InputDataSourceName.empty()) {
	_EnvelopeReader->AddTargetDataSource(InputDataSourceName);
    }
}

TKinokoEventProcessor::~TKinokoEventProcessor()
{
    delete _EnvelopeReader;
}

void TKinokoEventProcessor::SetLogger(TKinokoBuilderLogger* Logger)
{
    _SegmentProcessor->SetLogger(Logger);
    _EnvelopeReader->SetLogger(Logger);
}

void TKinokoEventProcessor::OnReceivePacket(void* Packet, long PacketSize)
{
    _EnvelopeReader->ProcessPacket(Packet, PacketSize);
}



TKinokoEventConsumer::TKinokoEventConsumer(const std::string& InputDataSourceName, TKinokoEventSegmentProcessor* SegmentProcessor)
{
    _SegmentProcessor = SegmentProcessor;
    _EnvelopeReader = new TKinokoEventEnvelopeReader(_SegmentProcessor);

    if (! InputDataSourceName.empty()) {
	_EnvelopeReader->AddTargetDataSource(InputDataSourceName);
    }
}

TKinokoEventConsumer::~TKinokoEventConsumer()
{
    delete _EnvelopeReader;
}

void TKinokoEventConsumer::SetLogger(TKinokoBuilderLogger* Logger)
{
    _SegmentProcessor->SetLogger(Logger);
    _EnvelopeReader->SetLogger(Logger);
}

void TKinokoEventConsumer::OnReceivePacket(void* Packet, long PacketSize)
{
    _EnvelopeReader->ProcessPacket(Packet, PacketSize);
}
