/* KinokoEventSorter.hh */
/* Created by Enomoto Sanshiro on 15 August 2009. */
/* Last updated by Enomoto Sanshiro on 7 March 2010. */


#ifndef __KinokoEventSorter_hh__
#define __KinokoEventSorter_hh__

#include <iostream>
#include <vector>
#include <deque>
#include <map>
#include <set>
#include "KinokoMemoryPool.hh"
#include "KinokoEventPiece.hh"
#include "KinokoChannelWatcher.hh"
#include "KinokoBuilderProcessor.hh"



class TKinokoEventSorterStatistics : public TKinokoEventPieceDefs {
  public:
    TKinokoEventSorterStatistics(void);
    virtual ~TKinokoEventSorterStatistics();
    virtual void ResetContext(void);
  protected:
    TEventKey _NumberOfBuiltEvents;
    TEventKey _NumberOfPieces;
    TEventKey _NumberOfEarlyArrivals, _NumberOfLateArrivals;
    TEventKey _NumberOfOutOfOrderArrivals;
    TEventKey _NumberOfCorruptedPieces;
    size_t _CorruptedPieceSize, _LateArrivalPieceSize;
  public:
    inline TEventKey NumberOfBuiltEvents(void) const {
	return _NumberOfBuiltEvents;
    }
    inline TEventKey NumberOfPieces(void) const {
	return _NumberOfPieces;
    }
    inline TEventKey NumberOfEarlyArrivals(void) const {
        return _NumberOfEarlyArrivals;
    }
    inline TEventKey NumberOfOutOfOrderArrivals(void) const {
        return _NumberOfOutOfOrderArrivals;
    }
    inline TEventKey NumberOfLateArrivals(void) const {
        return _NumberOfLateArrivals;
    }
    inline TEventKey NumberOfCorruptedPieces(void) const {
        return _NumberOfCorruptedPieces;
    }
    inline size_t CorruptedPieceSize(void) const {
        return _CorruptedPieceSize;
    }
    inline size_t LateArrivalPieceSize(void) const {
        return _LateArrivalPieceSize;
    }
};


class TKinokoEventSorterContext: public TKinokoEventSorterStatistics {
  public:
    TKinokoEventSorterContext(void);
    virtual ~TKinokoEventSorterContext();
    virtual void ResetContext(void);
    virtual int RegisterChannel(TChannelKey ChannelKey);
    virtual void DumpStatusTo(std::ostream& os);
    virtual void DumpChannelStatTo(std::ostream& os);
  protected:
    TEventKey _LastBuiltEventKey, _LastReceivedEventKey;
    TEventKey _NumberOfPendingEarlyArrivals;
    std::map<TEventKey, std::pair<int, size_t> > _HoldingEventSet;
    std::map<TChannelKey, int> _ChannelIndexTable;
    std::vector<TChannelKey> _ChannelKeyList;
    std::vector<TKinokoChannelWatcher> _ChannelWatcherList;
    std::vector<TEventKey> _CompleteEventKeyList;
    std::vector<std::deque<TEventKey> > _EventKeyQueueList;
    std::vector<std::set<TEventKey> > _EarlyArrivalEventKeySetList;
    std::vector<long> _ChannelEventCountList, _ChannelPieceCountList;
    std::vector<long> _LateArrivalLengthList;
  public:
    inline TEventKey LastBuiltEventKey(void) const {
	return _LastBuiltEventKey;
    }
    inline TEventKey LastEventKey(void) const {
	return _LastReceivedEventKey;
    }
    inline TEventKey NumberOfPendingEarlyArrivals(void) const {
        return _NumberOfPendingEarlyArrivals;
    }
    inline const std::map<TEventKey, std::pair<int, size_t> >& HoldingEventSet(void) const {
	return _HoldingEventSet;
    }
    inline const std::vector<TChannelKey>& ChannelKeyList(void) const {
	return _ChannelKeyList;
    }
    inline const std::vector<TKinokoChannelWatcher>& ChannelWatcherList(void) const {
	return _ChannelWatcherList;
    }
    inline const std::vector<TEventKey>& CompleteEventKeyList(void) const {
	return _CompleteEventKeyList;
    }
    inline const std::vector<std::deque<TEventKey> >& EventKeyQueueList(void) const {
	return _EventKeyQueueList;
    }
    inline const std::vector<std::set<TEventKey> >& EarlyArrivalEventKeySetList(void) const {
	return _EarlyArrivalEventKeySetList;
    }
    inline const std::vector<long>& ChannelEventCountList(void) const {
	return _ChannelEventCountList;
    }
    inline const std::vector<long>& ChannelPieceCountList(void) const {
	return _ChannelPieceCountList;
    }
};


class TKinokoEventSorter: public TKinokoEventPieceProcessor, public TKinokoEventSorterContext {
  public:
    TKinokoEventSorter(TKinokoEventPieceProcessor* OutputPieceProcessor, TKinokoMemoryPool* MemoryPool);
    virtual ~TKinokoEventSorter();
    virtual void ResetContext(void);
    virtual int RegisterChannel(TChannelKey ChannelKey);
    virtual const TKinokoMemoryPool& MemoryPool(void) const;
  public:
    virtual void ProcessRunBegin(void);
    virtual void ProcessRunEnd(void);
    virtual void Flush(void);
    virtual int ProcessPiece(const TKinokoEventPiece& EventPiece);
  public:
    virtual int BuildOne(TEventKey EventKey = -1);
    virtual int BuildReadies(void);
    virtual int BuildAll(void);
  protected:
    TKinokoMemoryPool* _MemoryPool;
    int _BufferReserveSize;
    TKinokoEventPieceProcessor* _OutputPieceProcessor;
    std::vector<std::deque<std::pair<void*, size_t> > > _EventPieceQueueList;
    std::vector<std::multimap<TEventKey, std::pair<void*, size_t> > > _EarlyArrivalPieceSetList;
};


#endif
