/* KinokoMemoryPool.cc */
/* Created by Enomoto Sanshiro on 26 November 2010. */
/* Updated by Enomoto Sanshiro on 26 November 2010. */


#include <iostream>
#include <sstream>
#include <iomanip>
#include <map>
#include "KinokoMemoryPool.hh"

using namespace std;


TKinokoMemoryPool::TKinokoMemoryPool(size_t Capacity)
{
    _Capacity = Capacity;
    _UsedSize = 0;
    _NumberOfEntries = 0;

    _MaximumUsage = 0;
}

TKinokoMemoryPool::~TKinokoMemoryPool()
{
}

void TKinokoMemoryPool::Initialize(void)
{
    _MaximumUsage = 0;
}

void TKinokoMemoryPool::Finalize(void)
{
    if (! _EntryTable.empty()) {
	cerr << "INTERNAL MEMORY ERROR: KinokoMemoryPool::Finalize(): ";
	cerr << "unreleased memory detected. releasing..." << endl;

        int Index = 0;
	for (
	    map<const void*, size_t>::iterator Entry = _EntryTable.begin();
	    Entry != _EntryTable.end();
	    Entry++
	){
	    cerr << "  " << Index++ << ": ";
	    cerr << "start: 0x" << hex << setw(8) << setfill('0');
            cerr << Entry->first << ", ";
            cerr << "size: " << dec << Entry->second << endl;
	    delete[] (Word*) (Entry->first);
	}
    }
}

void* TKinokoMemoryPool::Allocate(size_t Size)
{
    if (_UsedSize + Size > _Capacity) {
	return 0;
    }

    // for memory alignment //
    static const size_t WordSize = sizeof(Word);
    int NumberOfWords = Size / WordSize + 1;
    size_t AllocationSize = NumberOfWords * WordSize;

    void* Address;
    try {
	Address = new Word[NumberOfWords];
    }
    catch (std::bad_alloc& e) {
	Address = 0;
    }
    if (Address == 0) {
	return 0;
    }

    _NumberOfEntries++;
    _UsedSize += AllocationSize;
    _MaximumUsage = max(_MaximumUsage, _UsedSize);

    _EntryTable[Address] = AllocationSize;

    return Address;
}

void TKinokoMemoryPool::Release(const void* Address)
{
    map<const void*, size_t>::iterator Entry = _EntryTable.find(Address);
    if (Entry == _EntryTable.end()) {
	cerr << "INTERNAL MEMORY ERROR: KinokoMemoryPool::Release(): ";
	cerr << "Invalid address: " << Address << endl;
	return;
    }
	
    delete[] (Word*) Address;

    _UsedSize -= Entry->second;
    _NumberOfEntries--;
    _EntryTable.erase(Entry);
}

size_t TKinokoMemoryPool::Capacity(void) const
{
    return _Capacity;
}

size_t TKinokoMemoryPool::AvailableSize(void) const
{
    return _Capacity - _UsedSize;
}

int TKinokoMemoryPool::NumberOfEntries(void) const
{
    return _NumberOfEntries;
}

size_t TKinokoMemoryPool::UsedSize(void) const
{
    return _UsedSize;
}

size_t TKinokoMemoryPool::MaximumUsage(void) const
{
    return _MaximumUsage;
}
