/* KinokoVerticalBuilder.cc */
/* Created by Enomoto Sanshiro on 28 August 2009. */
/* Last updated by Enomoto Sanshiro on 8 May 2010. */


#include <string>
#include <iostream>
#include <vector>
#include "KinokoEventSorter.hh"
#include "KinokoSorterProcessor.hh"
#include "KinokoBuilder.hh"
#include "KinokoVerticalBuilder.hh"

using namespace std;


TKinokoBuilderDataSourceInlet::TKinokoBuilderDataSourceInlet(void)
{
}

TKinokoBuilderDataSourceInlet::~TKinokoBuilderDataSourceInlet()
{
}

void TKinokoBuilderDataSourceInlet::AddTargetDataSource(const std::string& DataSourceName, TKinokoEventPieceDefs::TStreamKey StreamKey)
{
    _InputDataSourceNameList.push_back(make_pair(DataSourceName, StreamKey));
}


TKinokoVerticalBuilder::TKinokoVerticalBuilder(void)
{
    _EnvelopeReader = 0;
    _Sorter = 0;
}

TKinokoVerticalBuilder::~TKinokoVerticalBuilder()
{
    delete _EnvelopeReader;
    delete _Sorter;
}

TKinokoEventSorter& TKinokoVerticalBuilder::BuilderConstruct(void) 
{
    _Sorter = new TKinokoEventSegmentSorter(_EnvelopeWriter, _MemoryPool);
    _Sorter->SetLogger(_Logger);

    _EnvelopeReader = new TKinokoEventEnvelopeReader(_Sorter);
    _EnvelopeReader->SetLogger(_Logger);

    for (unsigned i = 0; i < _InputDataSourceNameList.size(); i++) {
	_EnvelopeReader->AddTargetDataSource(
	    _InputDataSourceNameList[i].first,
	    _InputDataSourceNameList[i].second
	);
    }

    return _Sorter->Sorter();
}

void TKinokoVerticalBuilder::BuilderBegin(void)
{ 
    vector<pair<string, TStreamKey> > InputDataSourceNameList = (
	_EnvelopeReader->InputDataSourceNameList()
    );
    for (unsigned i = 0; i < InputDataSourceNameList.size(); i++) {
	TStreamKey StreamKey = InputDataSourceNameList[i].second; 
	_SorterController->RegisterStream(StreamKey);
    }
}

void TKinokoVerticalBuilder::BuilderEnd(void)
{ 
}

int TKinokoVerticalBuilder::BuilderProcess(void* Packet, long PacketSize)
{
    return _EnvelopeReader->ProcessPacket(Packet, PacketSize);
}
