/* KinokoStreamComponent.hh */
/* Created by Enomoto Sanshiro on 9 October 2000. */
/* Last updated by Enomoto Sanshiro on 21 September 2004. */


#ifndef __KinokoStreamComponent_hh__
#define __KinokoStreamComponent_hh__


#include <string>
#include "KinokoComponent.hh"
#include "KinokoPlatform.hh"
#include "KinokoLoggerOrb.hh"
#include "KinokoBufferServer.hh"


class TKinokoStreamComponent: public TKinokoComponent {
  public:
    TKinokoStreamComponent(const std::string& TypeName);
    virtual ~TKinokoStreamComponent();
    virtual void BuildDescriptor(TKcomComponentDescriptor& Descriptor);
    virtual int ProcessEvent(int EventId, TKcomEvent& Event, TKcomEventResponse& EventResponse);
    virtual void OnTransaction(void);
    virtual int DoTransaction(void);
    virtual void Initialize(void);
    virtual void Finalize(void);
  protected:
    TKinokoEventEmitter* _EventEmitter;
    TKinokoRegistry* _Registry;
    TKinokoLoggerProxy* _Logger;
  protected:
    enum TEventId {
	_NumberOfEvents = TKinokoComponent::_NumberOfEvents
    };
    enum TPropertyId {
	PropertyId_StreamType = TKinokoComponent::_NumberOfProperties,
	PropertyId_State,
	_NumberOfProperties
    };
    enum TState {
	State_Initial,
	State_ComponentReady,
	State_Connecting,
	State_StreamReady,
	State_SystemReady,
	State_Running,
	State_Shutdown,
	State_Error,
	_NumberOfStates
    };
    enum TStreamType {
	StreamType_Source,
	StreamType_Sink,
	StreamType_Pipe,
	StreamType_Buffer,
	_NumberOfStreamTypes
    };
    static const char* _StreamTypeName[_NumberOfStreamTypes];
  protected:
    TState CurrentState(void);
    void ChangeState(TState State);
    void ChangeStateNow(TState State);
  private:
    TState _State, _NextState;
};


class TKinokoLoggerBufferLogger: public TKinokoBufferLogger {
  public:
    TKinokoLoggerBufferLogger(const std::string& ComponentName, TKinokoLogger* Logger);
    virtual ~TKinokoLoggerBufferLogger();
    virtual int Write(const std::string& Message, int MessageLevel);
  protected:
    std::string _Name;
    TKinokoLogger* _Logger;
};


#endif
