/* KinokoHistogramView.cc */
/* Created by Enomoto Sanshiro on 25 January 2001. */
/* Last updated by Enomoto Sanshiro on 25 January 2001. */


#include <cmath>
#include "KameTabular.hh"
#include "KameRepository.hh"
#include "KinokoTabularView.hh"

using namespace std;
using namespace kame;


TKinokoTabularView::TKinokoTabularView(TKameTabular* Tabular, int NumberOfColumns)
{
    _Tabular = Tabular;
    _NumberOfColumns = NumberOfColumns;
}

TKinokoTabularView::~TKinokoTabularView()
{
    delete _Tabular;
}

void TKinokoTabularView::SaveThis(TKameRepository* Repository)
{
    try {
	Repository->SaveTabular(*_Tabular, _Name);
    }
    catch (TKameException &e) {
	throw TKinokoException(
	    "TKinokoTabularView::SaveThis()", e.Message()
	);
    }
}

void TKinokoTabularView::ClearThis(void)
{
    _Tabular->Clear();
}




TKinokoKoapTabularView::TKinokoKoapTabularView(TKameTabular* Tabular, int NumberOfColumns, ostream* OutputStream)
: TKinokoTabularView(Tabular, NumberOfColumns)
{
    _OutputStream = OutputStream;
}

TKinokoKoapTabularView::~TKinokoKoapTabularView()
{
}

void TKinokoKoapTabularView::DeployThis(void) 
{
    *_OutputStream << ".selectPage " << _PageNumber << ";" << endl;

    (*_OutputStream) << ".create textbox " << _Name << " ";
    (*_OutputStream) << 100 * Left() << " " << 100 * Top() << " ";
    (*_OutputStream) << 100 * Width() << " " << 100 * Height() << ";" << endl;

    (*_OutputStream) << _Name << " set title " << _Tabular->Title() << ";" << endl;
    (*_OutputStream) << _Name << " set columns " << _NumberOfColumns << ";" << endl;
    (*_OutputStream) << _Name << " drawframe;" << endl;
}

void TKinokoKoapTabularView::DrawThis(void)
{
    (*_OutputStream) << _Name << " clear;" << endl;
    (*_OutputStream) << _Name << " drawframe;" << endl;

    if (_Tabular->NumberOfRows() == 0) {
        (*_OutputStream) << _Name << " write ";
        (*_OutputStream) << "no data;" << endl;
	return;
    }

    int NumberOfFields = _Tabular->NumberOfRows();

    (*_OutputStream) << _Name << " write " << endl;
    for (int i = 0; i < NumberOfFields; i++) {
	(*_OutputStream) << (*_Tabular)[i][0] << ": ";
	(*_OutputStream) << (*_Tabular)[i][1] << endl;
    }
    (*_OutputStream) << ";" << endl;
}

void TKinokoKoapTabularView::ClearThis(void)
{
    TKinokoTabularView::ClearThis();
    
    (*_OutputStream) << _Name << " clear;" << endl;
    (*_OutputStream) << _Name << " drawframe;" << endl;
}
