/* KinokoViewAction.cc */
/* Created by Enomoto Sanshiro on 16 June 2002. */
/* Last updated by Enomoto Sanshiro on 29 September 2002. */


#include <string>
#include <map>
#include "MushMisc.hh"
#include "KinokoView.hh"
#include "KameRepository.hh"
#include "KinokoViewAction.hh"
#include "KinokoViewer.hh"

using namespace std;
using namespace kame;


TKinokoViewAction::TKinokoViewAction(void)
{
}

TKinokoViewAction::~TKinokoViewAction(void)
{
}



TKinokoViewDrawAction::TKinokoViewDrawAction(TKinokoView* View)
{
    _View = View;
}

TKinokoViewDrawAction::~TKinokoViewDrawAction()
{
}

void TKinokoViewDrawAction::Execute(void)
{
    _View->Draw();
}

void TKinokoViewDrawAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "Draw" << "\t" << _View->Name() << endl;
}



TKinokoViewClearAction::TKinokoViewClearAction(TKinokoView* View)
{
    _View = View;
}

TKinokoViewClearAction::~TKinokoViewClearAction()
{
}

void TKinokoViewClearAction::Execute(void)
{
    _View->Clear();
}

void TKinokoViewClearAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "Clear" << "\t" << _View->Name() << endl;
}



TKinokoViewSaveAction::TKinokoViewSaveAction(TKinokoView* View, TKameRepository* Repository)
{
    _View = View;
    _Repository = Repository;
}

TKinokoViewSaveAction::~TKinokoViewSaveAction()
{
}

void TKinokoViewSaveAction::Execute(void)
{
    _View->Save(_Repository);
}

void TKinokoViewSaveAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "Save" << "\t" << _View->Name() << endl;
}



TKinokoTrendViewDrawAction::TKinokoTrendViewDrawAction(TKinokoTrendView* TrendView, int FieldId)
: TKinokoViewDrawAction(TrendView)
{
    _TrendView = TrendView;
    _FieldId = FieldId;
}

TKinokoTrendViewDrawAction::~TKinokoTrendViewDrawAction()
{
}

void TKinokoTrendViewDrawAction::Execute(void)
{
    _TrendView->SetDisplayValue(_FieldId);
    TKinokoViewDrawAction::Execute();
}



TKinokoTrendViewSetOperationRangeAction::TKinokoTrendViewSetOperationRangeAction(TKinokoTrendView* TrendView, double LowerBound, double UpperBound, int NumberOfPointsToJudge)
{
    _TrendView = TrendView;

    _LowerBound = LowerBound;
    _UpperBound = UpperBound;
    _NumberOfPointsToJudge = NumberOfPointsToJudge;
}

TKinokoTrendViewSetOperationRangeAction::~TKinokoTrendViewSetOperationRangeAction()
{
}

void TKinokoTrendViewSetOperationRangeAction::Execute(void)
{
    _TrendView->SetOperationRange(
	_LowerBound, _UpperBound, _NumberOfPointsToJudge
   );
}

void TKinokoTrendViewSetOperationRangeAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "SetOperationRange" << "\t" << _TrendView->Name();
    os << ", " << _LowerBound << ", " << _UpperBound;
    os << ", " << _NumberOfPointsToJudge << endl;
}



TKinokoTrendViewEnableAlarmAction::TKinokoTrendViewEnableAlarmAction(TKinokoTrendView* TrendView, TKinokoEventEmitter* EventEmitter, const string& Message)
{
    _TrendView = TrendView;
    _EventEmitter = EventEmitter;
    _Message = Message;
}

TKinokoTrendViewEnableAlarmAction::~TKinokoTrendViewEnableAlarmAction()
{
}

void TKinokoTrendViewEnableAlarmAction::Execute(void)
{
    _TrendView->EnableAlarm(_EventEmitter, _Message);
}

void TKinokoTrendViewEnableAlarmAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "EnableAlarm" << "\t" << _TrendView->Name() << endl;
}



TKinokoTrendViewDisableAlarmAction::TKinokoTrendViewDisableAlarmAction(TKinokoTrendView* TrendView)
{
    _TrendView = TrendView;
}

TKinokoTrendViewDisableAlarmAction::~TKinokoTrendViewDisableAlarmAction()
{
}

void TKinokoTrendViewDisableAlarmAction::Execute(void)
{
    _TrendView->DisableAlarm();
}

void TKinokoTrendViewDisableAlarmAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "DisableAlarm" << "\t" << _TrendView->Name() << endl;
}


#if 1
//... for backward compatibility ...//
TKinokoHistoryViewDrawAction::TKinokoHistoryViewDrawAction(TKinokoHistoryView* HistoryView, int FieldId)
: TKinokoViewDrawAction(HistoryView)
{
    _HistoryView = HistoryView;
    _FieldId = FieldId;
}

TKinokoHistoryViewDrawAction::~TKinokoHistoryViewDrawAction()
{
}

void TKinokoHistoryViewDrawAction::Execute(void)
{
    _HistoryView->SetDisplayValue(_FieldId);
    TKinokoViewDrawAction::Execute();
}




TKinokoHistoryViewHoldAction::TKinokoHistoryViewHoldAction(TKinokoHistoryView* HistoryView, TKameHistory* History)
{
    _HistoryView = HistoryView;
    _History = History;
}

TKinokoHistoryViewHoldAction::~TKinokoHistoryViewHoldAction()
{
}

void TKinokoHistoryViewHoldAction::Execute(void)
{
    _History->HoldSample(TMushDateTime::SecSinceEpoch());
}

void TKinokoHistoryViewHoldAction::Dump(std::ostream& os, const std::string& Indent)
{
    os << Indent << "Hold" << "\t" << _HistoryView->Name() << endl;
}



TKinokoHistoryViewSetOperationRangeAction::TKinokoHistoryViewSetOperationRangeAction(TKinokoHistoryView* HistoryView, double LowerBound, double UpperBound)
{
    _HistoryView = HistoryView;
    _LowerBound = LowerBound;
    _UpperBound = UpperBound;
}

TKinokoHistoryViewSetOperationRangeAction::~TKinokoHistoryViewSetOperationRangeAction()
{
}

void TKinokoHistoryViewSetOperationRangeAction::Execute(void)
{
    _HistoryView->SetOperationRange(_LowerBound, _UpperBound);
}

void TKinokoHistoryViewSetOperationRangeAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "SetOperationRange" << "\t" << _HistoryView->Name();
    os << ", " << _LowerBound << ", " << _UpperBound << endl;
}



TKinokoHistoryViewEnableAlarmAction::TKinokoHistoryViewEnableAlarmAction(TKinokoHistoryView* HistoryView, TKinokoEventEmitter* EventEmitter, const string& Message)
{
    _HistoryView = HistoryView;
    _EventEmitter = EventEmitter;
    _Message = Message;
}

TKinokoHistoryViewEnableAlarmAction::~TKinokoHistoryViewEnableAlarmAction()
{
}

void TKinokoHistoryViewEnableAlarmAction::Execute(void)
{
    _HistoryView->EnableAlarm(_EventEmitter, _Message);
}

void TKinokoHistoryViewEnableAlarmAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "EnableAlarm" << "\t" << _HistoryView->Name() << endl;
}



TKinokoHistoryViewDisableAlarmAction::TKinokoHistoryViewDisableAlarmAction(TKinokoHistoryView* HistoryView)
{
    _HistoryView = HistoryView;
}

TKinokoHistoryViewDisableAlarmAction::~TKinokoHistoryViewDisableAlarmAction()
{
}

void TKinokoHistoryViewDisableAlarmAction::Execute(void)
{
    _HistoryView->DisableAlarm();
}

void TKinokoHistoryViewDisableAlarmAction::Dump(ostream& os, const std::string& Indent)
{
    os << Indent << "DisableAlarm" << "\t" << _HistoryView->Name() << endl;
}

#endif



#include <KinokoAnalysisAction.hh>
#include <KinokoViewSequence.hh>


TKinokoAnalysisDispatchAction::TKinokoAnalysisDispatchAction(TKinokoViewSequenceTable* ViewSequenceTable, const std::string& EventName, const std::vector<std::string>& EventArgumentList)
{
    _ViewSequenceTable = ViewSequenceTable;
    _EventName = EventName;
    _EventArgumentList = EventArgumentList;
}

TKinokoAnalysisDispatchAction::~TKinokoAnalysisDispatchAction()
{
}

void TKinokoAnalysisDispatchAction::Execute(void)
{
    //... _EventArgumentList is not implemented ...//
    if (! _ViewSequenceTable->ExecuteCommandSequence(_EventName)) {
	throw TKinokoException(
	    "unknown view sequence (command): " + _EventName
	);
    }
}

void TKinokoAnalysisDispatchAction::Dump(std::ostream& os, const std::string& Indent)
{
    os << Indent << "Dispatch  " << "\t" << _EventName;
    for (unsigned int i = 0; i < _EventArgumentList.size(); i++) {
	os << ", \"" << _EventArgumentList[i] << "\"";
    }
    os << endl;
}
