/* KinokoViewer.hh */
/* Created by Enomoto Sanshiro on 31 October 2000. */
/* Last updated by Enomoto Sanshiro on 20 September 2001. */


#ifndef __KinokoViewer_hh__
#define __KinokoViewer_hh__


#include <iostream>
#include <string>
#include <vector>
#include "MushMisc.hh"
#include "KinokoPlatform.hh"
#include "KinokoDataProcessor.hh"
#include "KinokoDataDistributor.hh"
#include "KinokoDataAnalyzer.hh"
#include "KinokoAnalysisSequenceBuilder.hh"
#include "KinokoView.hh"
#include "KinokoViewFactory.hh"
#include "KinokoViewCanvas.hh"
#include "KinokoViewSequence.hh"
#include "KinokoViewBuilder.hh"
#include "KinokoViewScript.hh"


class TKinokoViewer: public TKinokoDataConsumer {
  public:
    TKinokoViewer(void);
    virtual ~TKinokoViewer();
    virtual void OnConstruct(void);
    virtual void OnDestruct(void);
    virtual void OnRunBegin(void);
    virtual void OnRunEnd(void);
    virtual void OnReceiveDataPacket(void* DataPacket, long PacketSize);
    virtual void OnReceiveTrailerPacket(void* DataPacket, long PacketSize);
    virtual void OnReceiveCommandPacket(void* DataPacket, long PacketSize);
    virtual int ProcessData(void);
  public:
    virtual void SetScript(const std::string& ScriptFileName, const std::string& DisplayEntryName = "");
    virtual int ExecuteCommand(const std::string& Command);
    virtual void Reset(void);
    virtual void DrawAll(void);
    virtual int SaveImage(const std::string& ImageFileName);
    virtual int SaveDataContents(const std::string& DataFileName);
  protected:
    std::string _ScriptFileName;
    std::string _DisplayEntryName;
    std::ostream* _OutputStream;
    TKinokoAnalysisSequenceBuilder* _AnalysisSequenceBuilder;
    TKinokoViewBuilder* _ViewBuilder;
    TKinokoViewFactory* _ViewFactory;
    TKinokoViewScript* _ViewScript;
    TKinokoViewCanvas* _ViewCanvas;
    TKinokoViewSequenceTable* _ViewSequenceTable;
    TKinokoDataDistributor* _DataDistributor;
    std::vector<TKinokoAnalysisSequence*> _AnalysisSequenceList;
    bool _IsFirstRun;
    bool _IsRunning;
};


#endif
