/* kdfdescribe.cc */
/* Created by Enomoto Sanshiro on 3 September 2014. */
/* Last updated by Enomoto Sanshiro on 3 September 2014. */

#include <cstdlib>
#include <iostream>
#include <string>
#include <sstream>
#include "MushArgumentList.hh"
#include "KameTableFormat.hh"
#include "KameTreeSerializer.hh"
#include "KinokoKdfReader.hh"

using namespace std;
using namespace kame;


int main(int argc, char** argv)
{
    TMushArgumentList ArgumentList(argc, argv);
    if (ArgumentList.NumberOfParameters() < 1) {
	cerr << "Usage: " << ArgumentList.ProgramName();
	cerr << " DataFileName" << endl;
        return EXIT_FAILURE;
    }
    string DataFileName = ArgumentList[0];
    string Format = ArgumentList["--format"];
    
    ostringstream os;

    try {
        TKinokoKdfReader KdfReader(DataFileName);
	os << "# KdfHeader:" << endl;
	KdfReader.StorageHeader()->WriteTo(os, "#   ");
	KdfReader.ProcessUntilRunBegin();
	KdfReader.DataDescriptor()->WriteKntTo(os, "# ");
    }
    catch (TKinokoException& e) {
	cerr << "ERROR: " << e << endl;
        return EXIT_FAILURE;
    }

    istringstream is(os.str());
    TKameNtuple Ntuple;
    try {
        TKameTableFormat().Read(Ntuple, is);
    }
    catch (TKameException &e) {
        cerr << "ERROR: " << e << endl;
        return -1;
    }

    if (Format == "ktf") {
        TKameKtfTreeSerializer(cout).Serialize(Ntuple.Tree());
    }
    else if (Format == "json") {
        TKameJsonTreeSerializer(cout).Serialize(Ntuple.Tree());
    }
    else if (Format == "xml") {
        TKameXmlTreeSerializer(cout).Serialize(Ntuple.Tree());
    }
    else if (Format == "plist") {
        TKamePlistTreeSerializer(cout).Serialize(Ntuple.Tree());
    }
    else {
        TKameKtfTreeSerializer(cout).Serialize(Ntuple.Tree());
    }

    return 0;
}
