/* knttreedump.cxx */
/* Created by Enomoto Sanshiro on 3 September 2014. */
/* Last updated by Enomoto Sanshiro on 3 September 2014. */

#include <string>
#include <cstring>
#include <iostream>
#include <fstream>
#include "KameTableFormat.hh"
#include "KameTreeSerializer.hh"

using namespace std;
using namespace kame;


int main(int argc, char** argv)
{
    string FileName, NodePath = "/", Format;
    for (int i = 1; i < argc; i++) {
        if (strncmp(argv[i], "--node=", 7) == 0) {
            NodePath = string(argv[i]).substr(7);
        }
        else if (strncmp(argv[i], "--format=", 9) == 0) {
            Format = string(argv[i]).substr(9);
        }
        else if (strncmp(argv[i], "--", 2) != 0) {
            FileName = argv[i];
        }
    }
    if (FileName.empty()) {
        cerr << "USAGE: " << argv[0];
        cerr << " [ --node=NODEPATH ] [ --format=FORMAT ] FILENAME" << endl;
	return -1;
    }

    istream* Input = 0;
    ifstream* File = 0;
    if (FileName == "-") {
        Input = &cin;
    }
    else {
        File = new ifstream(FileName.c_str());
        if (! *File) {
            cerr << "ERROR: unable to open file: " << FileName << endl;
            return -1;
        }
        Input = File;
    }

    TKameTree Tree;
    try {
        TKameNtuple Ntuple;
        TKameTableFormat format;
        format.SetCommentHeader('#');
        format.SetTreeIndent('\0');
        format.Read(Ntuple, *Input);
        Tree["/"] = Ntuple.Tree();
    }
    catch (TKameException &e) {
        cerr << "ERROR: " << e << endl;
        return -1;
    }

    delete File;

    if (Format == "json") {
        TKameJsonTreeSerializer(cout).Serialize(Tree[NodePath]);
    }
    else if (Format == "xml") {
        TKameXmlTreeSerializer(cout).Serialize(Tree[NodePath], "KameTree");
    }
    else if (Format == "plist") {
        TKamePlistTreeSerializer(cout).Serialize(Tree[NodePath]);
    }
    else {
        Tree[NodePath].Dump();
    }

    return 0;
}
