/* KinokoCanvasColorScale.hh */
/* Created by Enomoto Sanshiro on 14 January 2001. */
/* Last updated by Enomoto Sanshiro on 14 January 2001. */


#ifndef __KinokoCanvasColorScale_hh
#define __KinokoCanvasColorScale_hh


#include <iostream>
#include <string>
#include <vector>
#include "KinokoCanvas.hh"
#include "KinokoCanvasImageArea.hh"


class TKinokoCanvasColorScale {
  public:
    TKinokoCanvasColorScale(TKinokoCanvas* Canvas, int NumberOfColors);
    virtual ~TKinokoCanvasColorScale();
    virtual int ColorIndexOf(double Value);
    virtual int BackgroundColorIndex(void);
  public:
    virtual void SetBackgroundColor(int ColorIndex);
    virtual void Initialize(void);
  protected:
    virtual void CreateColorTable(void) = 0;
  protected:
    TKinokoCanvas* _Canvas;
    TKinokoCanvasImageArea* _ImageArea;
    int _NumberOfColors;
    int* _ColorTable;
    int _BackgroundColor;
};


class TKinokoCanvasColorScaleFactory {
  public:
    static TKinokoCanvasColorScale* Create(TKinokoCanvas* Canvas, const std::string& Name);
};


class TKinokoCanvasGrayColorScale: public TKinokoCanvasColorScale {
  public:
    TKinokoCanvasGrayColorScale(TKinokoCanvas* Canvas, int NumberOfColors = 256);
    virtual ~TKinokoCanvasGrayColorScale();
  protected:
    virtual void CreateColorTable(void);
};


class TKinokoCanvasGradientColorScale: public TKinokoCanvasColorScale {
  public:
    TKinokoCanvasGradientColorScale(TKinokoCanvas* Canvas, int NumberOfColors, const std::string& Type, const std::string& Parameters);
    virtual ~TKinokoCanvasGradientColorScale();
  protected:
    virtual void CreateColorTable(void);
  protected:
    std::vector<float> _PointX, _PointR, _PointG, _PointB;
};


class TKinokoCanvasRainbowColorScale: public TKinokoCanvasColorScale {
  public:
    TKinokoCanvasRainbowColorScale(TKinokoCanvas* Canvas, int NumberOfColors = 256);
    virtual ~TKinokoCanvasRainbowColorScale();
    static void HSV2RGB(float h, float s, float v, float& r, float& g, float& b);
  protected:
    virtual void CreateColorTable(void);
};


class TKinokoCanvasParulaColorScale: public TKinokoCanvasColorScale {
  public:
    TKinokoCanvasParulaColorScale(TKinokoCanvas* Canvas, int NumberOfColors = 256);
    virtual ~TKinokoCanvasParulaColorScale();
    static void Value2RGB(float v, float& r, float& g, float& b);
  protected:
    virtual void CreateColorTable(void);
};


#endif
