/* KinokoCanvasSchematicObject.hh */
/* Created by Enomoto Sanshiro on 24 September. */
/* Last updated by Enomoto Sanshiro on 24 September. */


#ifndef __KinokoCanvasSchematicObject_hh
#define __KinokoCanvasSchematicObject_hh


#include <iostream>
#include <string>
#include <map>
#include "KinokoCanvasPictureObject.hh"
#include "KinokoCanvasColorScale.hh"
#include "KinokoCanvasDataScale.hh"


class TKinokoCanvasSchematicItem;
class TKinokoCanvasSchematicScale;
class TKinokoCanvasSchematicScaleTable;


class TKinokoCanvasSchematicObject: public TKinokoCanvasPictureObject {
  public:
    TKinokoCanvasSchematicObject(TKinokoCanvas* Canvas);
    virtual ~TKinokoCanvasSchematicObject();
    virtual TKinokoShellObject* Clone(void);
    virtual void Initialize(void);
    virtual int ProcessRangeSelect(int X0, int Y0, int X1, int Y1);
  public:
    virtual TKinokoCanvasSchematicScale* GetScale(const std::string& Name);
    virtual TKinokoCanvasImageArea* GetImageArea(void);
    virtual TKinokoCanvasTypesetter* GetTypesetter(void);
    virtual TKinokoCanvasMathTextComposer* GetMathTextComposer(void);
  protected:
    virtual int ProcessCommand(const std::string& Command, std::istream& InputStream);
    virtual int ProcessCreateScaleCommand(std::istream& InputStream);
    virtual int ProcessCreateItemCommand(std::istream& InputStream);
    virtual int ProcessSetScaleRangeCommand(std::istream& InputStream);
    virtual int ProcessSetScaleLogCommand(std::istream& InputStream);
    virtual int ProcessSetScaleTitleCommand(std::istream& InputStream);
    virtual int ProcessSetValueCommand(std::istream& InputStream);
    virtual int ProcessDrawScaleCommand(std::istream& InputStream);
    virtual int ProcessDrawTitleCommand(std::istream& InputStream);
  protected:
    std::map<std::string, TKinokoCanvasSchematicScale*> _ScaleTable;
    std::map<int, TKinokoCanvasSchematicItem*> _ItemTable;
    std::map<std::string, TKinokoCanvasSchematicItem*> _ItemPrototypeTable;
};


class TKinokoCanvasSchematicScale: public TKinokoCanvasColorScale {
  public:
    TKinokoCanvasSchematicScale(TKinokoCanvasColorScale* ColorScale, double Min, double Max);
    virtual ~TKinokoCanvasSchematicScale();
    virtual int ColorIndexOf(double Value);
    virtual void Initialize(void);
    virtual void SetRange(double Min, double Max);
    virtual void SetLog(bool IsLog = true);
    virtual void SetTitle(const std::string& Title);
    virtual void Draw(TKinokoCanvasSchematicObject* SchematicObject, float x0, float y0, float x1, float y1);
  protected:
    virtual void CreateColorTable(void) {}
  protected:
    double _Min, _Max, _Width;
    double _LogMin, _LogMax, _LogWidth;
    bool _IsLog;
    std::string _Title;
    TKinokoCanvasColorScale* _ColorScale;
    TKinokoCanvasDataScale* _DataScale;
};


class TKinokoCanvasSchematicItem {
  public:
    TKinokoCanvasSchematicItem(void);
    virtual ~TKinokoCanvasSchematicItem();
    virtual void Attach(TKinokoCanvasSchematicObject* SchematicObject);
    virtual TKinokoCanvasSchematicItem* Clone(void) = 0;
    virtual int Construct(std::istream& InputStream) = 0;
    virtual int SetValue(std::istream& InputStream) = 0;
    virtual bool Includes(int X, int Y) = 0;
  protected:
    inline float CanvasXOf(float x);
    inline float CanvasYOf(float y);
  protected:
    TKinokoCanvasSchematicObject* _SchematicObject;
    TKinokoCanvasImageArea* _ImageArea;
};


class TKinokoCanvasSchematicTextItem: public TKinokoCanvasSchematicItem {
  public:
    TKinokoCanvasSchematicTextItem(void);
    virtual ~TKinokoCanvasSchematicTextItem();
    virtual TKinokoCanvasSchematicItem* Clone(void);
    virtual int Construct(std::istream& InputStream);
    virtual int SetValue(std::istream& InputStream);
    virtual bool Includes(int X, int Y);
  protected:
    float _X0, _Y0, _X1, _Y1;
    std::string _Format;
    int _Color;
  private:
    int _BufferSize;
    char* _Buffer;
};


class TKinokoCanvasSchematicRectangleItem: public TKinokoCanvasSchematicItem {
  public:
    TKinokoCanvasSchematicRectangleItem(void);
    virtual ~TKinokoCanvasSchematicRectangleItem();
    virtual TKinokoCanvasSchematicItem* Clone(void);
    virtual int Construct(std::istream& InputStream);
    virtual int SetValue(std::istream& InputStream);
    virtual bool Includes(int X, int Y);
  protected:
    float _X0, _Y0, _X1, _Y1;
    TKinokoCanvasSchematicScale* _Scale;
};


class TKinokoCanvasSchematicCircleItem: public TKinokoCanvasSchematicItem {
  public:
    TKinokoCanvasSchematicCircleItem(void);
    virtual ~TKinokoCanvasSchematicCircleItem();
    virtual TKinokoCanvasSchematicItem* Clone(void);
    virtual int Construct(std::istream& InputStream);
    virtual int SetValue(std::istream& InputStream);
    virtual bool Includes(int X, int Y);
  protected:
    float _X0, _Y0, _Rx, _Ry;
    TKinokoCanvasSchematicScale* _Scale;
};



class TKinokoCanvasSchematicPieItem: public TKinokoCanvasSchematicItem {
  public:
    TKinokoCanvasSchematicPieItem(void);
    virtual ~TKinokoCanvasSchematicPieItem();
    virtual TKinokoCanvasSchematicItem* Clone(void);
    virtual int Construct(std::istream& InputStream);
    virtual int SetValue(std::istream& InputStream);
    virtual bool Includes(int X, int Y);
  protected:
    float _X0, _Y0, _R, _Theta0, _Theta1;
    TKinokoCanvasSchematicScale* _Scale;
    int _BorderColor;
};



inline float TKinokoCanvasSchematicItem::CanvasXOf(float x)
{
    return _SchematicObject->CanvasXOf(x);
}

inline float TKinokoCanvasSchematicItem::CanvasYOf(float y)
{
    return _SchematicObject->CanvasYOf(y);
}


#endif
